@@ -52,68 +52,78 @@
 #endif // _WIN32 || _WIN64
 
 // ----------------------------------------------------------------------------
-// test_node
+// test_download
 
-void  download_node (UgetNode* node, const UgetPluginInfo* info)
+void  download_by_plugin(UgData* data, const UgetPluginInfo* pinfo)
 {
 	UgetCommon*   common;
+	UgetFiles*    files;
 	UgetPlugin*   plugin;
 	UgetEvent*    events;
 	UgetEvent*    cur;
 	UgetEvent*    next;
 	UgetProgress* progress;
-	int           integer[2];
+	int           index;
+	int           speed_limit[2];
 	char*         string[5];
 
-	plugin = uget_plugin_new (info);
-//	integer[0] = 1000000;
-//	integer[1] = 1000000;
-//	uget_plugin_ctrl (plugin, UGET_PLUGIN_CTRL_SPEED, integer);
+	speed_limit[0] = 1000000;
+	speed_limit[1] = 1000000;
 
-	uget_plugin_start (plugin, node);
-	while (uget_plugin_sync (plugin)) {
-		ug_sleep (1000);
+	plugin = uget_plugin_new(pinfo);
+	uget_plugin_accept(plugin, data);
+	uget_plugin_ctrl(plugin, UGET_PLUGIN_CTRL_SPEED, speed_limit);
+	if (uget_plugin_start(plugin) == FALSE) {
+		uget_plugin_unref(plugin);
+		puts("plug-in failed to start.");
+		return;
+	}
+
+	while (uget_plugin_sync(plugin, data)) {
+		ug_sleep(1000);
 		// event
-		events = uget_plugin_pop (plugin);
+		events = uget_plugin_pop(plugin);
 		for (cur = events;  cur;  cur = next) {
 			next = cur->next;
 			printf ("\n" "Event type: %d - %s\n", cur->type, cur->string);
-			uget_event_free (cur);
+			uget_event_free(cur);
 		}
 		// progress
-		progress = ug_info_get (&node->info, UgetProgressInfo);
+		progress = ug_data_get(data, UgetProgressInfo);
 		if (progress == NULL)
 			continue;
-		string[0] = ug_str_from_int_unit (progress->complete, NULL);
-		string[1] = ug_str_from_int_unit (progress->total, NULL);
-		string[2] = ug_str_from_int_unit (progress->uploaded, NULL);
-		string[3] = ug_str_from_int_unit (progress->download_speed, "/s");
-		string[4] = ug_str_from_int_unit (progress->upload_speed, "/s");
-		printf ("\r" "DL: %s / %s, %d%%, %s | UL: %s, %s" "      ",
-				string[0], string[1], progress->percent,
-				string[3], string[2], string[4]);
-		for (integer[0] = 0;  integer[0] < 5;  integer[0]++)
-			ug_free (string[integer[0]]);
+		string[0] = ug_str_from_int_unit(progress->complete, NULL);
+		string[1] = ug_str_from_int_unit(progress->total, NULL);
+		string[2] = ug_str_from_int_unit(progress->uploaded, NULL);
+		string[3] = ug_str_from_int_unit(progress->download_speed, "/s");
+		string[4] = ug_str_from_int_unit(progress->upload_speed, "/s");
+		printf("\r" "DL: %s / %s, %d%%, %s | UL: %s, %s" "      ",
+		       string[0], string[1], progress->percent,
+		       string[3], string[2], string[4]);
+		for (index = 0;  index < 5;  index++)
+			ug_free(string[index]);
 	}
 	// these data may changed, print them.
-	common = ug_info_get (&node->info, UgetCommonInfo);
-	printf ("\n"
-			"node->name : %s\n"
-			"common->uri : %s\n"
-			"common->file : %s\n",
-			node->name, common->uri, common->file);
+	common = ug_data_get(data, UgetCommonInfo);
+	printf("\n"
+	       "common->name : %s\n"
+	       "common->uri : %s\n"
+	       "common->file : %s\n",
+	       common->name, common->uri, common->file);
 	// print children
-	for (node = node->children;  node;  node = node->next)
-		printf ("child node name : %s\n", node->name);
+	files = ug_data_realloc(data, UgetFilesInfo);
+	for (index=0; index < files->collection.length; index++) {
+		printf("file name : %s\n", files->collection.at[index].path);
+	}
 
-	uget_plugin_unref (plugin);
+	uget_plugin_unref(plugin);
 }
 
-void  test_node_download (void)
+void  test_download(void)
 {
 	UgetCommon*  common;
 	UgetHttp*    http;
-	UgetNode*    node;
+	UgData*      data;
 	char*        referrer;
 	char*        uri;
 	char*        mirrors;
@@ -128,12 +138,12 @@ void  test_node_download (void)
 //	referrer = "http://code.google.com/p/tortoisegit/wiki/Download?tm=2";
 	referrer = NULL;
 
-	node = uget_node_new (NULL);
+	data = ug_data_new(8, 0);
 	// commom options
-	common = ug_info_realloc (&node->info, UgetCommonInfo);
-	common->uri = ug_strdup (uri);
+	common = ug_data_realloc(data, UgetCommonInfo);
+	common->uri = ug_strdup(uri);
 	if (mirrors)
-		common->mirrors = ug_strdup (mirrors);
+		common->mirrors = ug_strdup(mirrors);
 //	common->folder = ug_strdup ("D:\\Downloads");
 //	common->max_connections = 2;
 	common->debug_level = 1;
@@ -141,15 +151,15 @@ void  test_node_download (void)
 	common->connect_timeout = 30;
 
 	// http options
-	http = ug_info_realloc (&node->info, UgetHttpInfo);
+	http = ug_data_realloc(data, UgetHttpInfo);
 	if (referrer)
-		http->referrer = ug_strdup (referrer);
+		http->referrer = ug_strdup(referrer);
 
-//	download_node (node, UgetPluginCurlInfo);
-//	download_node (node, UgetPluginAria2Info);
-//	download_node (node, UgetPluginMediaInfo);
-	download_node (node, UgetPluginMegaInfo);
-	uget_node_unref (node);
+//	download_by_plugin(data, UgetPluginCurlInfo);
+//	download_by_plugin(data, UgetPluginAria2Info);
+//	download_by_plugin(data, UgetPluginMediaInfo);
+	download_by_plugin(data, UgetPluginMegaInfo);
+	ug_data_unref(data);
 }
 
 // ----------------------------------------------------------------------------
@@ -160,32 +170,32 @@ void  test_setup_plugin_aria2 (void)
 	const UgetPluginInfo* pinfo;
 
 	pinfo = UgetPluginAria2Info;
-	uget_plugin_set (pinfo, UGET_PLUGIN_INIT, (void*) TRUE);
-	uget_plugin_set (pinfo, UGET_PLUGIN_ARIA2_URI,
+	uget_plugin_global_set(pinfo, UGET_PLUGIN_GLOBAL_INIT, (void*) TRUE);
+	uget_plugin_global_set(pinfo, UGET_PLUGIN_ARIA2_GLOBAL_URI,
 			"http://localhost/jsonrpc");
 #if defined _WIN32 || defined _WIN64
-	uget_plugin_set (pinfo, UGET_PLUGIN_ARIA2_PATH,
+	uget_plugin_global_set(pinfo, UGET_PLUGIN_ARIA2_GLOBAL_PATH,
 			"C:\\Program Files\\uGet\\bin\\aria2c.exe");
 #endif
-	uget_plugin_set (pinfo, UGET_PLUGIN_ARIA2_ARGUMENT,
+	uget_plugin_global_set(pinfo, UGET_PLUGIN_ARIA2_GLOBAL_ARGUMENT,
 			"--enable-rpc=true -D --check-certificate=false");
-	uget_plugin_set (pinfo, UGET_PLUGIN_ARIA2_LAUNCH, (void*) TRUE);
-	uget_plugin_set (pinfo, UGET_PLUGIN_ARIA2_SHUTDOWN, (void*) TRUE);
+	uget_plugin_global_set(pinfo, UGET_PLUGIN_ARIA2_GLOBAL_LAUNCH, (void*) TRUE);
+	uget_plugin_global_set(pinfo, UGET_PLUGIN_ARIA2_GLOBAL_SHUTDOWN, (void*) TRUE);
 	ug_sleep (1000);
-	uget_plugin_set (pinfo, UGET_PLUGIN_INIT, (void*) FALSE);
+	uget_plugin_global_set(pinfo, UGET_PLUGIN_GLOBAL_INIT, (void*) FALSE);
 	ug_sleep (1000);
 }
 
 // ----------------------------------------------------------------------------
 // test_task
 
-void  print_node_speed_limit (UgetNode** dnode, int count)
+void  print_speed_limit (UgetNode** dnode, int count)
 {
 	int           total[2] = {0,0};
 	UgetRelation* relation;
 
 	for (count = 0;  count < 7;  count++) {
-		relation = ug_info_get (&dnode[count]->info, UgetRelationInfo);
+		relation = ug_data_get (dnode[count]->data, UgetRelationInfo);
 		printf ("limit D: %d, U: %d | speed D: %d, U: %d\n",
 				relation->task.limit[0],
 				relation->task.limit[1],
@@ -213,7 +223,7 @@ void  test_task (void)
 	// task speed control -------------------
 	for (count = 0;  count < 7;  count++) {
 		dnode[count] = uget_node_new (NULL);
-		relation = ug_info_realloc (&dnode[count]->info, UgetRelationInfo);
+		relation = ug_data_realloc (dnode[count]->data, UgetRelationInfo);
 		relation->task.limit[0] = 2000;
 		relation->task.limit[1] = 1500;
 		relation->task.speed[0] = 2000 - count * 200;
@@ -224,17 +234,17 @@ void  test_task (void)
 	relation->task.limit[1] = 0;
 
 	uget_task_set_speed (task, 12000, 8000);
-	print_node_speed_limit(dnode, 7);
+	print_speed_limit(dnode, 7);
 	puts ("---");
 	uget_task_adjust_speed (task);
-	print_node_speed_limit(dnode, 7);
+	print_speed_limit(dnode, 7);
 
 	uget_task_remove_all (task);
 	uget_task_final (task);
 	free (task);
 
 	for (count = 0;  count < 7;  count++)
-		uget_node_unref (dnode[count]);
+		uget_node_free (dnode[count]);
 }
 
 // ----------------------------------------------------------------------------
@@ -247,12 +257,12 @@ void  setup_app (UgetApp* app)
 	UgetCommon*    common;
 
 	cnode = uget_node_new (NULL);
-	cnode->name = ug_strdup ("Home Category");
-	category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+	category = ug_data_realloc (cnode->data, UgetCategoryInfo);
 	*(char**)ug_array_alloc (&category->schemes, 1) = ug_strdup ("http");
 	*(char**)ug_array_alloc (&category->schemes, 1) = ug_strdup ("https");
 	*(char**)ug_array_alloc (&category->schemes, 1) = ug_strdup ("ftp");
-	common = ug_info_realloc (&cnode->info, UgetCommonInfo);
+	common = ug_data_realloc (cnode->data, UgetCommonInfo);
+	common->name = ug_strdup ("Home Category");
 	common->max_connections = 2;
 	uget_app_add_category (app, cnode, TRUE);
 
@@ -269,8 +279,8 @@ void  start_app (UgetApp* app)
 	char*          string[2];
 
 	dnode = uget_node_new (NULL);
-	dnode->name = ug_strdup ("Download");
-	common = ug_info_realloc (&dnode->info, UgetCommonInfo);
+	common = ug_data_realloc (dnode->data, UgetCommonInfo);
+	common->name = ug_strdup ("Download");
 	common->uri = ug_strdup ("http://www.utorrent.com/scripts/dl.php?track=stable&build=29812&client=utorrent");
 	common->folder = ug_strdup ("D:\\Downloads");
 	common->debug_level = 1;
@@ -309,7 +319,7 @@ void  test_app_node (UgetApp* app)
 
 	for (count = 0;  count < 7;  count++) {
 		dnode[count] = uget_node_new (NULL);
-		common = ug_info_realloc (&dnode[count]->info, UgetCommonInfo);
+		common = ug_data_realloc (dnode[count]->data, UgetCommonInfo);
 		common->uri = ug_strdup ("ftp://127.0.0.1/");
 		common->folder = ug_strdup ("D:\\Downloads");
 		common->keeping.enable = TRUE;
@@ -361,22 +371,22 @@ void  test_app (void)
 int   main (void)
 {
 	// initialize plug-in
-	uget_plugin_set (UgetPluginCurlInfo, UGET_PLUGIN_INIT, (void*) TRUE);
-	uget_plugin_set (UgetPluginAria2Info, UGET_PLUGIN_INIT, (void*) TRUE);
-	uget_plugin_set (UgetPluginMediaInfo, UGET_PLUGIN_INIT, (void*) TRUE);
-	uget_plugin_set (UgetPluginMegaInfo, UGET_PLUGIN_INIT, (void*) TRUE);
-//	test_setup_plugin_aria2 ();
+	uget_plugin_global_set(UgetPluginCurlInfo, UGET_PLUGIN_GLOBAL_INIT, (void*) TRUE);
+	uget_plugin_global_set(UgetPluginAria2Info, UGET_PLUGIN_GLOBAL_INIT, (void*) TRUE);
+	uget_plugin_global_set(UgetPluginMediaInfo, UGET_PLUGIN_GLOBAL_INIT, (void*) TRUE);
+	uget_plugin_global_set(UgetPluginMegaInfo, UGET_PLUGIN_GLOBAL_INIT, (void*) TRUE);
+//	test_setup_plugin_aria2();
 
-	test_node_download ();
-//	test_task ();
-//	test_app ();
+	test_download();
+//	test_task();
+//	test_app();
 
-//	uget_plugin_set (UgetPluginAria2Info, UGET_PLUGIN_ARIA2_SHUTDOWN_NOW, (void*) TRUE);
+//	uget_plugin_global_set(UgetPluginAria2Info, UGET_PLUGIN_ARIA2_GLOBAL_SHUTDOWN_NOW, (void*) TRUE);
 	// finalize plug-in
-	uget_plugin_set (UgetPluginCurlInfo, UGET_PLUGIN_INIT, (void*) FALSE);
-	uget_plugin_set (UgetPluginAria2Info, UGET_PLUGIN_INIT, (void*) FALSE);
-	uget_plugin_set (UgetPluginMediaInfo, UGET_PLUGIN_INIT, (void*) FALSE);
-	uget_plugin_set (UgetPluginMegaInfo, UGET_PLUGIN_INIT, (void*) FALSE);
+	uget_plugin_global_set(UgetPluginCurlInfo, UGET_PLUGIN_GLOBAL_INIT, (void*) FALSE);
+	uget_plugin_global_set(UgetPluginAria2Info, UGET_PLUGIN_GLOBAL_INIT, (void*) FALSE);
+	uget_plugin_global_set(UgetPluginMediaInfo, UGET_PLUGIN_GLOBAL_INIT, (void*) FALSE);
+	uget_plugin_global_set(UgetPluginMegaInfo, UGET_PLUGIN_GLOBAL_INIT, (void*) FALSE);
 	ug_sleep(1000);
 
 	return 0;
