@@ -38,6 +38,11 @@
 #include <string.h>
 #include <UgArray.h>
 #include <UgetNode.h>
+
+#include <UgJson.h>
+#include <UgString.h>
+#include <UgGroupData.h>
+#include <UgetFiles.h>
 //#include <UgetPlugin.h>
 
 #include <UgetA2cf.h>
@@ -62,18 +67,14 @@ void init_node (UgetNode* parent)
 	UgetNode*  newtwo;
 
 	newone = uget_node_new (NULL);
-	newone->type = 1;
 	uget_node_append (parent, newone);
 	newtwo = uget_node_new (NULL);
-	newtwo->type = 2;
 	uget_node_append (parent, newtwo);
 
 	parent = newtwo;
 	newone = uget_node_new (NULL);
-	newone->type = 3;
 	uget_node_append (parent, newone);
 	newtwo = uget_node_new (NULL);
-	newtwo->type = 4;
 	uget_node_append (parent, newtwo);
 }
 
@@ -134,7 +135,7 @@ void test_fake_path ()
 
 	print_fake_path (node);
 
-	uget_node_unref (node);
+	uget_node_free (node);
 }
 
 void parse_node (UgetNode* parent)
@@ -472,6 +473,89 @@ void test_seq (void)
 	uget_sequence_final (&useq);
 }
 
+// ----------------------------------------------------------------------------
+// UgetFiles
+
+void test_files_json(UgetFiles* files)
+{
+	UgJson       json;
+	UgBuffer     buffer;
+	UgJsonError  code;
+	UgetFiles*   files2;
+
+	ug_json_init(&json);
+	ug_buffer_init(&buffer, 4096);
+
+	// write
+	ug_json_begin_write(&json, UG_JSON_FORMAT_INDENT, &buffer);
+	ug_json_write_object_head(&json);
+    ug_json_write_entry(&json, files, files->info->entry);
+	ug_json_write_object_tail(&json);
+	ug_json_end_write(&json);
+	ug_buffer_write_char(&buffer, '\0');
+	puts("\n--- print files ---");
+	puts(buffer.beg);
+
+	files2 = ug_group_data_new(UgetFilesInfo);
+	// parse
+	ug_json_begin_parse(&json);
+	ug_json_push(&json, ug_json_parse_entry, files2, (void*)files2->info->entry);
+	ug_json_push(&json, ug_json_parse_object, NULL, NULL);
+	code = ug_json_parse(&json, buffer.beg, -1);
+	ug_json_end_parse(&json);
+	printf("\n" "parse return %d\n", code);
+
+	// write
+	buffer.cur = buffer.beg;
+	ug_json_begin_write(&json, UG_JSON_FORMAT_INDENT, &buffer);
+    ug_json_write_entry(&json, files2, files2->info->entry);
+	ug_json_end_write(&json);
+	ug_buffer_write_char(&buffer, '\0');
+	puts("\n--- print files2 ---");
+	puts(buffer.beg);
+
+	ug_group_data_free(files2);
+
+	ug_buffer_clear(&buffer, TRUE);
+	ug_json_final(&json);
+}
+
+void test_files(void)
+{
+	UgetFiles* files;
+	UgetFiles* src;
+	UgetFile*  element;
+
+	files = ug_group_data_new(UgetFilesInfo);
+	src   = ug_group_data_new(UgetFilesInfo);
+
+	element = uget_files_realloc(files, "0.mp4");
+	element = uget_files_realloc(files, "1.mp4");
+	element = uget_files_realloc(files, "2.mp4");
+
+	test_files_json(files);
+
+	element = uget_files_realloc(src, "0.mp4");
+	element = uget_files_realloc(src, "1.mp4");
+	element = uget_files_realloc(src, "2.mp4");
+	element = uget_files_realloc(src, "3.mp4");
+
+	element = uget_files_realloc(src, "foo.mp4");
+	element = uget_files_realloc(src, "xxx.apk");
+
+	uget_files_sync(files, src);
+	test_files_json(files);
+
+	element->state |= UGET_FILE_STATE_DELETED;
+	src->sync_count++;
+
+	uget_files_sync(files, src);
+	test_files_json(files);
+
+	ug_group_data_free(files);
+	ug_group_data_free(src);
+}
+
 // ----------------------------------------------------------------------------
 // main
 
@@ -484,7 +568,8 @@ int main (void)
 //	test_uget_curl ();
 //	test_uget_rss ();
 //	test_media ();
-	test_seq ();
+//	test_seq ();
+	test_files();
 
 	return 0;
 }
