@@ -61,35 +61,65 @@
 #define _(x)   x
 #endif
 
-static struct UgetNodeNotification  notification_real =
-		{NULL, NULL, NULL, NULL,
-		(UgCompareFunc) NULL, FALSE,
-		NULL};
-static struct UgetNodeNotification  notification_split =
-		{uget_node_create_split, NULL, NULL, NULL,
-		(UgCompareFunc) NULL, FALSE,
-		NULL};
-static struct UgetNodeNotification  notification_sorted =
-		{uget_node_create_sorted, NULL, NULL, NULL,
-		(UgCompareFunc) NULL, FALSE,
-		NULL};
-static struct UgetNodeNotification  notification_sorted_split =
-		{uget_node_create_split, NULL, NULL, NULL,
-		(UgCompareFunc) NULL, FALSE,
-		NULL};
-static struct UgetNodeNotification  notification_mix =
-		{uget_node_create_mix, NULL, NULL, NULL,
-		(UgCompareFunc) NULL, FALSE,
-		NULL};
-static struct UgetNodeNotification  notification_mix_split =
-		{uget_node_create_mix_split, NULL, NULL, NULL,
-		(UgCompareFunc) NULL, FALSE,
-		NULL};
+static struct UgetNodeControl  control_real =
+{
+	NULL,                       // struct UgetNodeControl*  children;
+	{NULL, NULL, NULL},         // struct UgetNodeNotifier  notifier;
+	{NULL, FALSE},              // struct UgetNodeSort      sort;
+	NULL,                       // UgetNodeFunc             filter;
+	NULL                        // void*                    data;
+};
+
+static struct UgetNodeControl  control_split =
+{
+	NULL,                       // struct UgetNodeControl*  children;
+	{NULL, NULL, NULL},         // struct UgetNodeNotifier  notifier;
+	{NULL, FALSE},              // struct UgetNodeSort      sort;
+	uget_node_filter_split,     // UgetNodeFunc             filter;
+	NULL                        // void*                    data;
+};
+
+static struct UgetNodeControl  control_sorted =
+{
+	NULL,                       // struct UgetNodeControl*  children;
+	{NULL, NULL, NULL},         // struct UgetNodeNotifier  notifier;
+	{NULL, FALSE},              // struct UgetNodeSort      sort;
+	uget_node_filter_sorted,    // UgetNodeFunc             filter;
+	NULL                        // void*                    data;
+};
+
+static struct UgetNodeControl  control_sorted_split =
+{
+	NULL,                       // struct UgetNodeControl*  children;
+	{NULL, NULL, NULL},         // struct UgetNodeNotifier  notifier;
+	{NULL, FALSE},              // struct UgetNodeSort      sort;
+	uget_node_filter_split,     // UgetNodeFunc             filter;
+	NULL                        // void*                    data;
+};
+
+static struct UgetNodeControl  control_mix =
+{
+	NULL,                       // struct UgetNodeControl*  children;
+	{NULL, NULL, NULL},         // struct UgetNodeNotifier  notifier;
+	{NULL, FALSE},              // struct UgetNodeSort      sort;
+	uget_node_filter_mix,       // UgetNodeFunc             filter;
+	NULL                        // void*                    data;
+};
+
+static struct UgetNodeControl  control_mix_split =
+{
+	NULL,                       // struct UgetNodeControl*  children;
+	{NULL, NULL, NULL},         // struct UgetNodeNotifier  notifier;
+	{NULL, FALSE},              // struct UgetNodeSort      sort;
+	uget_node_filter_mix_split, // UgetNodeFunc             filter;
+	NULL                        // void*                    data;
+};
 
 
 void  uget_app_init (UgetApp* app)
 {
-	UgetNode*  node;
+	UgetCommon* common;
+	UgetNode*   node;
 
 	// real and virtual root nodes
 	uget_node_init (&app->real, NULL);
@@ -98,15 +128,16 @@ void  uget_app_init (UgetApp* app)
 	uget_node_init (&app->sorted_split, &app->sorted);
 	uget_node_init (&app->mix, &app->real);
 	uget_node_init (&app->mix_split, &app->mix);
-	app->real.notification = &notification_real;
-	app->split.notification = &notification_split;
-	app->sorted.notification = &notification_sorted;
-	app->sorted_split.notification = &notification_sorted_split;
-	app->mix.notification = &notification_mix;
-	app->mix_split.notification = &notification_mix_split;
+	app->real.control = &control_real;
+	app->split.control = &control_split;
+	app->sorted.control = &control_sorted;
+	app->sorted_split.control = &control_sorted_split;
+	app->mix.control = &control_mix;
+	app->mix_split.control = &control_mix_split;
 	// add virtual category - "All Category"
 	node = uget_node_new (NULL);
-	node->name = ug_strdup (_("All Category"));
+	common = ug_data_realloc(node->data, UgetCommonInfo);
+	common->name = ug_strdup (_("All Category"));
 	uget_node_append (&app->mix, node);
 
 	uget_task_init (&app->task);
@@ -118,6 +149,7 @@ void  uget_app_init (UgetApp* app)
 	// info registry
 	ug_registry_init (&app->infos);
 	ug_registry_add (&app->infos, UgetCommonInfo);
+	ug_registry_add (&app->infos, UgetFilesInfo);
 	ug_registry_add (&app->infos, UgetProgressInfo);
 	ug_registry_add (&app->infos, UgetProxyInfo);
 	ug_registry_add (&app->infos, UgetHttpInfo);
@@ -126,7 +158,7 @@ void  uget_app_init (UgetApp* app)
 	ug_registry_add (&app->infos, UgetRelationInfo);
 	ug_registry_add (&app->infos, UgetCategoryInfo);
 	ug_registry_sort (&app->infos);
-	ug_info_set_registry (&app->infos);
+	ug_data_set_registry (&app->infos);
 }
 
 void  uget_app_final (UgetApp* app)
@@ -135,11 +167,11 @@ void  uget_app_final (UgetApp* app)
 	uget_task_final (&app->task);
 	uget_app_clear_plugins (app);
 
-	uget_node_unref_children (&app->mix_split);
-	uget_node_unref_children (&app->mix);
-	uget_node_unref_children (&app->sorted);
-	uget_node_unref_children (&app->split);
-	uget_node_unref_children (&app->real);
+	uget_node_clear_children (&app->mix_split);
+	uget_node_clear_children (&app->mix);
+	uget_node_clear_children (&app->sorted);
+	uget_node_clear_children (&app->split);
+	uget_node_clear_children (&app->real);
 
 	ug_registry_final (&app->plugins);
 	ug_registry_final (&app->infos);
@@ -158,7 +190,7 @@ static UgArrayPtr* uget_app_store_nodes (UgetApp* app, UgetNode* parent)
 //	array->length = 0;
 	ug_array_alloc (array, parent->n_children);
 	for (index = 0, dnode = parent->children;  dnode;  dnode = dnode->next)
-		array->at[index++] = dnode->data;
+		array->at[index++] = dnode->base;
 
 	return array;
 }
@@ -170,6 +202,7 @@ static void uget_app_clear_nodes (UgetApp* app)
 
 static int  uget_app_activate (UgetApp* app, UgetNode* cnode, UgetCategory* category)
 {
+	UgetRelation* relation;
 	UgetNode*   dnode;
 	UgetNode*   sibling;
 	UgetLog*    log;
@@ -183,12 +216,13 @@ static int  uget_app_activate (UgetApp* app, UgetNode* cnode, UgetCategory* cate
 	for (index = 0;  index < array->length;  index++) {
 		dnode = array->at[index];
 		uget_node_updated (dnode);
-		if (dnode->state & UGET_STATE_ACTIVE) {
+		relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+		if (relation->group & UGET_GROUP_ACTIVE) {
 			// remove node and insert it again to sort node
-			if (app->mix.notification->compare) {
+			if (app->mix.control->sort.compare) {
 				sibling = dnode->next;
 				uget_node_remove (cnode, dnode);
-				uget_node_unref_fake (dnode);
+				uget_node_clear_fake (dnode);
 				uget_node_insert (cnode, sibling, dnode);
 				app->n_moved++;
 			}
@@ -197,19 +231,19 @@ static int  uget_app_activate (UgetApp* app, UgetNode* cnode, UgetCategory* cate
 
 		uget_task_remove (&app->task, dnode);
 		uget_node_remove (cnode, dnode);
-		uget_node_unref_fake (dnode);
-		if (dnode->state & UGET_STATE_COMPLETED) {
-			dnode->state |= UGET_STATE_FINISHED;
+		uget_node_clear_fake (dnode);
+		if (relation->group & UGET_GROUP_COMPLETED) {
+			relation->group |= UGET_GROUP_FINISHED;
 			sibling = category->finished->children;
 			// completed time
-			log = ug_info_realloc (&dnode->info, UgetLogInfo);
+			log = ug_data_realloc (dnode->data, UgetLogInfo);
 			log->completed_time = time(NULL);    // get current time
 			app->n_completed++;
 		}
 		else {
-			dnode->state |= UGET_STATE_QUEUING;
+			relation->group |= UGET_GROUP_QUEUING;
 			sibling = category->queuing->children;
-			if (dnode->state & UGET_STATE_ERROR)
+			if (relation->group & UGET_GROUP_ERROR)
 				app->n_error++;
 		}
 
@@ -231,6 +265,7 @@ static int  uget_app_activate (UgetApp* app, UgetNode* cnode, UgetCategory* cate
 
 static void uget_app_queuing (UgetApp* app, UgetNode* cnode, UgetCategory* category)
 {
+	UgetRelation* relation;
 	UgetNode*   dnode;
 	UgArrayPtr* array;
 	int         index;
@@ -243,7 +278,8 @@ static void uget_app_queuing (UgetApp* app, UgetNode* cnode, UgetCategory* categ
 		dnode = array->at[index];
 		if (category->active->n_children >= category->active_limit)
 			break;
-		if (dnode->state & UGET_STATE_INACTIVE)
+		relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+		if (relation->group & UGET_GROUP_INACTIVE)
 			continue;
 		uget_app_activate_download (app, dnode);
 		app->n_moved++;
@@ -257,37 +293,50 @@ int  uget_app_grow (UgetApp* app, int no_queuing)
 {
 	UgetCategory*  category;
 	UgetNode*      cnode;
-	UgetNode*      dnode;
 	int            n_active = 0;
 
 	// dispatch plug-in event, calc speed
 	uget_task_dispatch (&app->task);
 	// active, queuing, finished, recycled
 	for (cnode = app->real.children;  cnode;  cnode = cnode->next) {
-		category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+		category = ug_data_realloc (cnode->data, UgetCategoryInfo);
 		if (category == NULL)
 			continue;
 		uget_app_activate (app, cnode, category);
 		if (no_queuing == FALSE)
 			uget_app_queuing (app, cnode, category);
+		n_active += category->active->n_children;
+	}
+	return n_active;
+}
+
+int   uget_app_trim(UgetApp* app)
+{
+	UgetCategory*  category;
+	UgetNode*      cnode;
+	UgetNode*      dnode;
+	int            n_deleted_prev = app->n_deleted;
+
+	for (cnode = app->real.children;  cnode;  cnode = cnode->next) {
+		category = ug_data_realloc(cnode->data, UgetCategoryInfo);
+		if (category == NULL)
+			continue;
 		while (category->finished->n_children > category->finished_limit) {
 			dnode = category->finished->last->real;
-			uget_uri_hash_remove_download (app->uri_hash, dnode);
-			uget_node_remove (cnode, dnode);
-			uget_node_unref (dnode);
+			uget_uri_hash_remove_download(app->uri_hash, dnode->data);
+			uget_node_remove(cnode, dnode);
+			uget_node_free(dnode);
 			app->n_deleted++;
 		}
 		while (category->recycled->n_children > category->recycled_limit) {
 			dnode = category->recycled->last->real;
-			uget_uri_hash_remove_download (app->uri_hash, dnode);
-			uget_node_remove (cnode, dnode);
-			uget_node_unref (dnode);
+			uget_uri_hash_remove_download(app->uri_hash, dnode->data);
+			uget_node_remove(cnode, dnode);
+			uget_node_free(dnode);
 			app->n_deleted++;
 		}
-		n_active += category->active->n_children;
 	}
-
-	return n_active;
+	return app->n_deleted - n_deleted_prev;
 }
 
 void  uget_app_set_config_dir (UgetApp* app, const char* dir)
@@ -302,12 +351,12 @@ void  uget_app_set_sorting (UgetApp* app, UgCompareFunc compare, int reversed)
 	UgetNode*  real;
 
 	node = app->mix.children;
-	if (app->mix.notification->reversed != reversed) {
-		app->mix.notification->reversed  = reversed;
-		app->mix_split.notification->reversed = reversed;
-		app->sorted.notification->reversed = reversed;
-		app->sorted_split.notification->reversed = reversed;
-		if (app->mix.notification->compare == compare && compare) {
+	if (app->mix.control->sort.reverse != reversed) {
+		app->mix.control->sort.reverse  = reversed;
+		app->mix_split.control->sort.reverse = reversed;
+		app->sorted.control->sort.reverse = reversed;
+		app->sorted_split.control->sort.reverse = reversed;
+		if (app->mix.control->sort.compare == compare && compare) {
 			// reverse first category in app->mix
 			ug_node_reverse ((UgNode*) node);
 			// reverse each category in app->mix_split
@@ -323,11 +372,11 @@ void  uget_app_set_sorting (UgetApp* app, UgCompareFunc compare, int reversed)
 		}
 	}
 
-	if (app->mix.notification->compare != compare) {
-		app->mix.notification->compare  = compare;
-		app->mix_split.notification->compare = compare;
-		app->sorted.notification->compare = compare;
-		app->sorted_split.notification->compare = compare;
+	if (app->mix.control->sort.compare != compare) {
+		app->mix.control->sort.compare  = compare;
+		app->mix_split.control->sort.compare = compare;
+		app->sorted.control->sort.compare = compare;
+		app->sorted_split.control->sort.compare = compare;
 		if (compare == NULL) {
 			// reorder first category in app->mix
 			for (real = app->real.last;  real;  real = real->prev)
@@ -366,35 +415,35 @@ void  uget_app_set_notification (UgetApp* app, void* data,
                                  UgetNodeFunc removed,
                                  UgNotifyFunc updated)
 {
-	notification_real.inserted = inserted;
-	notification_real.removed  = removed;
-	notification_real.updated  = updated;
-	notification_real.data     = data;
+	control_real.notifier.inserted = inserted;
+	control_real.notifier.removed  = removed;
+	control_real.notifier.updated  = updated;
+	control_real.data              = data;
 
-	notification_split.inserted = inserted;
-	notification_split.removed  = removed;
-	notification_split.updated  = updated;
-	notification_split.data     = data;
+	control_split.notifier.inserted = inserted;
+	control_split.notifier.removed  = removed;
+	control_split.notifier.updated  = updated;
+	control_split.data              = data;
 
-	notification_sorted.inserted = inserted;
-	notification_sorted.removed  = removed;
-	notification_sorted.updated  = updated;
-	notification_sorted.data     = data;
+	control_sorted.notifier.inserted = inserted;
+	control_sorted.notifier.removed  = removed;
+	control_sorted.notifier.updated  = updated;
+	control_sorted.data              = data;
 
-	notification_sorted_split.inserted = inserted;
-	notification_sorted_split.removed  = removed;
-	notification_sorted_split.updated  = updated;
-	notification_sorted_split.data     = data;
+	control_sorted_split.notifier.inserted = inserted;
+	control_sorted_split.notifier.removed  = removed;
+	control_sorted_split.notifier.updated  = updated;
+	control_sorted_split.data              = data;
 
-	notification_mix.inserted = inserted;
-	notification_mix.removed  = removed;
-	notification_mix.updated  = updated;
-	notification_mix.data     = data;
+	control_mix.notifier.inserted = inserted;
+	control_mix.notifier.removed  = removed;
+	control_mix.notifier.updated  = updated;
+	control_mix.data              = data;
 
-	notification_mix_split.inserted = inserted;
-	notification_mix_split.removed  = removed;
-	notification_mix_split.updated  = updated;
-	notification_mix_split.data     = data;
+	control_mix_split.notifier.inserted = inserted;
+	control_mix_split.notifier.removed  = removed;
+	control_mix_split.notifier.updated  = updated;
+	control_mix_split.data              = data;
 }
 
 void  uget_app_add_category (UgetApp* app, UgetNode* cnode, int save_file)
@@ -404,25 +453,24 @@ void  uget_app_add_category (UgetApp* app, UgetNode* cnode, int save_file)
 	char*          path_base;
 	char*          path;
 
-	cnode->type = UGET_NODE_CATEGORY;
 	uget_node_append (&app->real, cnode);
 	uget_uri_hash_add_category (app->uri_hash, cnode);
-	category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+	category = ug_data_realloc (cnode->data, UgetCategoryInfo);
 	for (node = cnode->fake;  node;  node = node->peer) {
-		switch (node->state) {
-		case UGET_STATE_ACTIVE:
+		switch (uget_node_get_group(node)) {
+		case UGET_GROUP_ACTIVE:
 			category->active = node;
 			break;
 
-		case UGET_STATE_QUEUING:
+		case UGET_GROUP_QUEUING:
 			category->queuing = node;
 			break;
 
-		case UGET_STATE_FINISHED:
+		case UGET_GROUP_FINISHED:
 			category->finished = node;
 			break;
 
-		case UGET_STATE_RECYCLED:
+		case UGET_GROUP_RECYCLED:
 			category->recycled = node;
 			break;
 
@@ -506,7 +554,7 @@ void  uget_app_delete_category (UgetApp* app, UgetNode* cnode)
 	uget_app_stop_category (app, cnode);
 	uget_uri_hash_remove_category (app->uri_hash, cnode);
 	uget_node_remove (&app->real, cnode);
-	uget_node_unref (cnode);
+	uget_node_free (cnode);
 
 	if (app->config_dir == NULL)
 		path_base = ug_strdup ("category");
@@ -544,7 +592,7 @@ void  uget_app_stop_category (UgetApp* app, UgetNode* cnode)
 	UgArrayPtr*    array;
 	int            index;
 
-	category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+	category = ug_data_realloc (cnode->data, UgetCategoryInfo);
 
 	// Because uget_app_queue_download() will change node linking,
 	// program must store active nodes to array before calling uget_app_queue_download()
@@ -566,7 +614,7 @@ void  uget_app_pause_category (UgetApp* app, UgetNode* cnode)
 	UgArrayPtr*    array;
 	int            index;
 
-	category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+	category = ug_data_realloc (cnode->data, UgetCategoryInfo);
 
 	// Because uget_app_queue_download() will change node linking,
 	// program must store active nodes to array before calling uget_app_queue_download()
@@ -600,7 +648,7 @@ void  uget_app_resume_category (UgetApp* app, UgetNode* cnode)
 	UgArrayPtr*    array;
 	int            index;
 
-	category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+	category = ug_data_realloc (cnode->data, UgetCategoryInfo);
 
 	// Because uget_app_queue_download() will change node linking,
 	// program must store active nodes to array before calling uget_app_queue_download()
@@ -652,7 +700,7 @@ UgetNode* uget_app_match_category (UgetApp* app, UgUri* uuri, const char* file)
 	matched.count = 0;
 
 	for (cnode = app->real.children;  cnode;  cnode = cnode->next) {
-		category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+		category = ug_data_realloc (cnode->data, UgetCategoryInfo);
 		if (category == NULL)
 			continue;
 		// null-terminated
@@ -690,13 +738,15 @@ int  uget_app_add_download_uri (UgetApp* app, const char* uri, UgetNode* cnode,
 	UgetCommon* common;
 
 	dnode = uget_node_new (NULL);
-	common = ug_info_realloc (&dnode->info, UgetCommonInfo);
+	common = ug_data_realloc (dnode->data, UgetCommonInfo);
 	common->uri = ug_strdup (uri);
 	return uget_app_add_download (app, dnode, cnode, apply);
 }
 
 int  uget_app_add_download (UgetApp* app, UgetNode* dnode, UgetNode* cnode, int apply)
 {
+	UgetRelation* relation;
+	UgetRelation* relation_c;
 	UgetNode*     sibling;
 	UgetLog*      log;
 	UgUri         uuri;
@@ -706,19 +756,19 @@ int  uget_app_add_download (UgetApp* app, UgetNode* dnode, UgetNode* cnode, int
 		UgetCategory* category;
 	} temp;
 
-	temp.common = ug_info_realloc (&dnode->info, UgetCommonInfo);
+	temp.common = ug_data_realloc (dnode->data, UgetCommonInfo);
 	// replace invalid characters \/:*?"<>| by _ in filename.
 	if (temp.common->file)
 		ug_str_replace_chars (temp.common->file, "\\/:*?\"<>|", '_');
 	// decode name, filename, and category
 	if (temp.common->uri) {
 		ug_uri_init (&uuri, temp.common->uri);
-		// assign node name if it's name is NULL
-		if (dnode->name == NULL) {
+		// set UgetCommon::name if it's name is NULL
+		if (temp.common->name == NULL) {
 			if (temp.common->file)
-				dnode->name = ug_strdup (temp.common->file);
+				temp.common->name = ug_strdup(temp.common->file);
 			else
-				uget_node_set_name_by_uri (dnode, &uuri);
+				temp.common->name = uget_name_from_uri(&uuri);
 		}
 		if (cnode == NULL)
 			cnode = uget_app_match_category (app, &uuri, temp.common->file);
@@ -727,21 +777,22 @@ int  uget_app_add_download (UgetApp* app, UgetNode* dnode, UgetNode* cnode, int
 	if (cnode == NULL)
 		cnode = app->real.children;
 	if (cnode) {
-		dnode->type   = UGET_NODE_DOWNLOAD;
-		dnode->state &= UGET_STATE_CATEGORY | UGET_STATE_PAUSED;
-		dnode->state |= UGET_STATE_QUEUING;
-		log = ug_info_realloc (&dnode->info, UgetLogInfo);
+		relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+		relation->group &= UGET_GROUP_MAJOR | UGET_GROUP_PAUSED;
+		relation->group |= UGET_GROUP_QUEUING;
+		log = ug_data_realloc (dnode->data, UgetLogInfo);
 		log->added_time = time (NULL);    // get current time
 		if (apply) {
 			value = temp.common->keeping.enable;
 			temp.common->keeping.enable = TRUE;
 			temp.common->keeping.uri = TRUE;
-			ug_info_assign (&dnode->info, &cnode->info, UgetCategoryInfo);
+			ug_data_assign (dnode->data, cnode->data, UgetCategoryInfo);
 			temp.common->keeping.enable = value;
-			if (cnode->state & UGET_STATE_PAUSED)
-				dnode->state |= UGET_STATE_PAUSED;
+			relation_c = ug_data_realloc(cnode->data, UgetRelationInfo);
+			if (relation_c->group & UGET_GROUP_PAUSED)
+				relation->group |= UGET_GROUP_PAUSED;
 		}
-		temp.category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+		temp.category = ug_data_realloc (cnode->data, UgetCategoryInfo);
 		// try to insert download before finished and recycled
 		sibling = temp.category->finished->children;
 		if (sibling == NULL)
@@ -750,7 +801,7 @@ int  uget_app_add_download (UgetApp* app, UgetNode* dnode, UgetNode* cnode, int
 		if (sibling)
 			sibling = sibling->real;
 		uget_node_insert (cnode, sibling, dnode);
-		uget_uri_hash_add_download (app->uri_hash, dnode);
+		uget_uri_hash_add_download(app->uri_hash, dnode->data);
 		return TRUE;
 	}
 	return FALSE;
@@ -777,13 +828,15 @@ int   uget_app_move_download_to (UgetApp* app, UgetNode* dnode, UgetNode* cnode)
 {
 	UgetNode*      sibling;
 	UgetCategory*  category;
+	UgetRelation*  relation;
 
 	if (dnode->parent == cnode)
 		return FALSE;
-	category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+	category = ug_data_realloc(cnode->data, UgetCategoryInfo);
+	relation = ug_data_realloc(dnode->data, UgetRelationInfo);
 
-	switch (dnode->state & UGET_STATE_CATEGORY) {
-	case UGET_STATE_ACTIVE:
+	switch (relation->group & UGET_GROUP_MAJOR) {
+	case UGET_GROUP_ACTIVE:
 		sibling = category->queuing->children;
 		if (sibling == NULL)
 			sibling = category->finished->children;
@@ -792,14 +845,14 @@ int   uget_app_move_download_to (UgetApp* app, UgetNode* dnode, UgetNode* cnode)
 		break;
 
 	default:
-	case UGET_STATE_QUEUING:
-	case UGET_STATE_FINISHED:
+	case UGET_GROUP_QUEUING:
+	case UGET_GROUP_FINISHED:
 		sibling = category->finished->children;
 		if (sibling == NULL)
 			sibling = category->recycled->children;
 		break;
 
-	case UGET_STATE_RECYCLED:
+	case UGET_GROUP_RECYCLED:
 		sibling = category->recycled->children;
 		break;
 	}
@@ -807,96 +860,77 @@ int   uget_app_move_download_to (UgetApp* app, UgetNode* dnode, UgetNode* cnode)
 		sibling = sibling->real;
 
 	uget_node_remove (dnode->parent, dnode);
-	uget_node_unref_fake (dnode);
+	uget_node_clear_fake (dnode);
 	uget_node_insert (cnode, sibling, dnode);
 	return TRUE;
 }
 
-static int  delete_dnode_files (UgetNode* dnode, int  has_aria2_file)
+// used by uget_app_delete_download()
+static int  delete_files(UgetFiles* files, int has_temp_file)
 {
-	UgetNode*  fnode;   // filenode
-	UgetNode*  fnode_next;
-	int        error;
-	int        error_count = 0;
+	UgetFile* file1;
+	int  n_error;
+	int  index;
 
-	if (has_aria2_file == TRUE) {
-		// move first aria2 to tail
-		for (fnode = dnode->children;  fnode;  fnode = fnode_next) {
-			fnode_next = fnode->next;
-			if (strstr (fnode->name, ".aria2")) {
-				uget_node_move (dnode, NULL, fnode);
-				break;
-			}
-		}
+	for (n_error = 0, index = 0;  index < files->collection.length;  index++) {
+		file1 = files->collection.at + index;
+		if (file1->state & UGET_FILE_STATE_DELETED)
+			continue;
+
+		if (ug_file_is_exist(file1->path) == FALSE)
+			file1->state |= UGET_FILE_STATE_DELETED;
+		else if (ug_remove(file1->path) != 0)
+			n_error++;
+		else if (file1->type != UGET_FILE_TEMPORARY)
+			file1->state |= UGET_FILE_STATE_DELETED;
 	}
 
-	for (fnode = dnode->children;  fnode;  fnode = fnode_next) {
-		fnode_next = fnode->next;
-
-		if (fnode->name == NULL) {
-			uget_node_remove (dnode, fnode);
-			continue;
-		}
-		if (ug_file_is_exist (fnode->name) == FALSE) {
-			uget_node_remove (dnode, fnode);
-			continue;
-		}
-
-		error = ug_remove (fnode->name);
-		if (error != 0)
-			error_count++;
-		else if (error_count == 0 || strstr (fnode->name, ".aria2") == NULL)
-			uget_node_remove (dnode, fnode);
-	}
-
-	if (dnode->children == NULL)
-		return TRUE;
-	else
-		return FALSE;
+	return (n_error == 0) ? TRUE : FALSE;
 }
 
-static UG_THREAD_RETURN_TYPE  delete_file_thread (UgetNode* dnode)
+// used by uget_app_delete_download()
+static UgThreadResult  delete_files_thread(UgetFiles* files)
 {
 	int  count;
 
 #ifdef USE__ANDROID__SAF
-	if (delete_dnode_files (dnode, TRUE) == FALSE)
+	if (delete_files(files, TRUE) == FALSE)
 #endif
 	for (count = 0;  count < 3;  count++) {
-		ug_sleep (2 * 1000);    // sleep 2 seconds
-		delete_dnode_files (dnode, FALSE);
-		if (dnode->children == NULL)
+		ug_sleep(2 * 1000);    // sleep 2 seconds
+		if (delete_files(files, FALSE))
 			break;
 	}
 
-	uget_node_unref (dnode);
-	return UG_THREAD_RETURN_VALUE;
+	ug_group_data_free(files);
+	return UG_THREAD_RESULT;
 }
 
 int  uget_app_delete_download (UgetApp* app, UgetNode* dnode, int delete_file)
 {
-	UgetNode*  cnode;
 	UgThread   thread;
+	UgetFiles* files;
 	int        is_active;
 
-	cnode = dnode->parent;
-	is_active = uget_task_remove (&app->task, dnode);
-	uget_node_remove (cnode, dnode);
-	uget_node_unref_fake (dnode);
-	uget_uri_hash_remove_download (app->uri_hash, dnode);
-
-	if (delete_file == TRUE) {
+	is_active = uget_task_remove(&app->task, dnode);
 #ifdef USE__ANDROID__SAF
-		is_active = TRUE;  // delete files in thread if program use Android SAF
+	is_active = TRUE;  // delete files in thread if program use Android SAF
 #endif
-		if (is_active == TRUE || delete_dnode_files (dnode, TRUE) == FALSE) {
-			ug_thread_create (&thread, (UgThreadFunc) delete_file_thread, dnode);
-			ug_thread_unjoin (&thread);
+	uget_uri_hash_remove_download(app->uri_hash, dnode->data);
+	files = ug_data_set(dnode->data, UgetFilesInfo, NULL);
+	uget_node_free(dnode);
+
+	if (delete_file == TRUE && files) {
+		if (is_active == TRUE || delete_files(files, TRUE) == FALSE) {
+			// delete files and free 'files' in thread
+			ug_thread_create(&thread, (UgThreadFunc) delete_files_thread, files);
+			ug_thread_unjoin(&thread);
 			return FALSE;
 		}
 	}
 
-	uget_node_unref (dnode);
+	if (files)
+		ug_group_data_free(files);
 	return TRUE;
 }
 
@@ -905,24 +939,26 @@ int  uget_app_recycle_download (UgetApp* app, UgetNode* dnode)
 	UgetNode*  cnode;
 	UgetNode*  sibling;
 	UgetCategory* category;
+	UgetRelation* relation;
 
 	cnode = dnode->parent;
 	uget_task_remove (&app->task, dnode);
 	uget_node_remove (cnode, dnode);
 
-	if (dnode->state & UGET_STATE_RECYCLED) {
-		uget_node_unref (dnode);
+	relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+	if (relation->group & UGET_GROUP_RECYCLED) {
+		uget_node_free (dnode);
 		return FALSE;
 	}
 	else {
-		dnode->state &= ~UGET_STATE_CATEGORY;
-		dnode->state |=  UGET_STATE_RECYCLED;
-		uget_node_unref_fake (dnode);
-		category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+		relation->group &= ~UGET_GROUP_MAJOR;
+		relation->group |=  UGET_GROUP_RECYCLED;
+		uget_node_clear_fake (dnode);
+		category = ug_data_realloc (cnode->data, UgetCategoryInfo);
 		// try to insert download before recycled
 		sibling = category->recycled->children;
 		if (sibling)
-			sibling = sibling->data;
+			sibling = sibling->base;
 		uget_node_insert (cnode, sibling, dnode);
 		return TRUE;
 	}
@@ -932,6 +968,7 @@ int   uget_app_activate_download (UgetApp* app, UgetNode* dnode)
 {
 	UgetLog*         log;
 	UgetCommon*      common;
+	UgetRelation*    relation;
 	UgetNode*        cnode;
 	UgetNode*        sibling;
 	union {
@@ -939,18 +976,19 @@ int   uget_app_activate_download (UgetApp* app, UgetNode* dnode)
 		UgetPluginInfo*  pinfo;
 	} temp;
 
-	if (dnode->state & UGET_STATE_ACTIVE)
+	relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+	if (relation->group & UGET_GROUP_ACTIVE)
 		return FALSE;
-	common = ug_info_get (&dnode->info, UgetCommonInfo);
+	common = ug_data_get (dnode->data, UgetCommonInfo);
 	if (common == NULL || common->uri == NULL)
 		return FALSE;
 	// match plug-in
-	log = ug_info_realloc (&dnode->info, UgetLogInfo);
+	log = ug_data_realloc (dnode->data, UgetLogInfo);
 	temp.pinfo = uget_app_match_plugin (app, common->uri, NULL);
 	if (temp.pinfo == NULL) {
 		// no plug-in support
 		uget_app_queue_download (app, dnode);
-		dnode->state |= UGET_STATE_ERROR;
+		relation->group |= UGET_GROUP_ERROR;
 		ug_list_prepend (&log->messages,
 				(UgLink*) uget_event_new_error (
 						UGET_EVENT_ERROR_UNSUPPORTED_SCHEME, NULL));
@@ -970,15 +1008,15 @@ int   uget_app_activate_download (UgetApp* app, UgetNode* dnode)
 	if (uget_task_add (&app->task, dnode, temp.pinfo) == FALSE) {
 		// plug-in start failed.
 		uget_app_queue_download (app, dnode);
-		dnode->state |=  UGET_STATE_ERROR;
+		relation->group |=  UGET_GROUP_ERROR;
 		uget_node_updated (dnode);
 		return FALSE;
 	}
 	// change node state and move node position
 	uget_node_remove (cnode, dnode);
-	uget_node_unref_fake (dnode);
-	dnode->state =  UGET_STATE_ACTIVE;
-	temp.category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+	uget_node_clear_fake (dnode);
+	relation->group =  UGET_GROUP_ACTIVE;
+	temp.category = ug_data_realloc (cnode->data, UgetCategoryInfo);
 	// try to insert download before queuing, finished, and recycled
 	sibling = temp.category->queuing->children;
 	if (sibling == NULL)
@@ -996,23 +1034,25 @@ int   uget_app_pause_download (UgetApp* app, UgetNode* dnode)
 {
 #if 0
 	UgetCategory* category;
+	UgetRelation* relation;
 	UgetNode*     sibling;
 	UgetNode*     cnode;
 	UgetNode*     fake;
 
-	if (dnode->state & UGET_STATE_UNRUNNABLE)
+	relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+	if (relation->group & UGET_GROUP_UNRUNNABLE)
 		return FALSE;
 	uget_task_remove (&app->task, dnode);
-	dnode->state |= UGET_STATE_PAUSED;
+	relation->group |= UGET_GROUP_PAUSED;
 
 	cnode = dnode->parent;
-	category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+	category = ug_data_realloc (cnode->data, UgetCategoryInfo);
 	for (fake = dnode->fake;  fake;  fake = fake->peer) {
 		if (fake->parent == category->active) {
 			uget_node_remove (cnode, dnode);
-			uget_node_unref_fake (dnode);
+			uget_node_clear_fake (dnode);
 
-			dnode->state |= UGET_STATE_QUEUING;
+			relation->group |= UGET_GROUP_QUEUING;
 			// try to insert download before queuing, finished, and recycled
 			sibling = category->queuing->children;
 			if (sibling == NULL)
@@ -1027,17 +1067,18 @@ int   uget_app_pause_download (UgetApp* app, UgetNode* dnode)
 		}
 	}
 #else
-	if (dnode->state & UGET_STATE_ACTIVE) {
+	UgetRelation* relation;
+
+	relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+	if (relation->group & UGET_GROUP_ACTIVE) {
 //		uget_task_remove (&app->task, dnode);
-		UgetRelation*  relation;
-		relation = ug_info_get (&dnode->info, UgetRelationInfo);
 		if (relation && relation->task.plugin)
 			uget_plugin_stop (relation->task.plugin);
-		dnode->state &= ~UGET_STATE_ACTIVE;
+		relation->group &= ~UGET_GROUP_ACTIVE;
 	}
-	else if (dnode->state & UGET_STATE_UNRUNNABLE)
+	else if (relation->group & UGET_GROUP_UNRUNNABLE)
 		return FALSE;
-	dnode->state |= UGET_STATE_PAUSED;
+	relation->group |= UGET_GROUP_PAUSED;
 #endif
 	return TRUE;
 }
@@ -1047,24 +1088,26 @@ int   uget_app_queue_download (UgetApp* app, UgetNode* dnode)
 	UgetNode*     cnode;
 	UgetNode*     sibling;
 	UgetCategory* category;
+	UgetRelation* relation;
 
-	if ((dnode->state & UGET_STATE_ACTIVE)     == 0 &&
-		(dnode->state & UGET_STATE_UNRUNNABLE) == 0)
+	relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+	if ((relation->group & UGET_GROUP_ACTIVE)     == 0 &&
+		(relation->group & UGET_GROUP_UNRUNNABLE) == 0)
 		return FALSE;
 
-	if (dnode->state & UGET_STATE_QUEUING)
-		dnode->state = UGET_STATE_QUEUING;
+	if (relation->group & UGET_GROUP_QUEUING)
+		relation->group = UGET_GROUP_QUEUING;
 	else {
 		cnode = dnode->parent;
 		uget_node_remove (cnode, dnode);
-		uget_node_unref_fake (dnode);
+		uget_node_clear_fake (dnode);
 
 		// --- decide sibling position & insert before it ---
 		// if current download is in active, insert it before queuing,
 		// otherwise insert it before finished and recycled.
-		category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+		category = ug_data_realloc (cnode->data, UgetCategoryInfo);
 		sibling = NULL;
-		if (dnode->state & UGET_STATE_ACTIVE) {
+		if (relation->group & UGET_GROUP_ACTIVE) {
 			uget_task_remove (&app->task, dnode);
 			sibling = category->queuing->children;
 		}
@@ -1075,7 +1118,7 @@ int   uget_app_queue_download (UgetApp* app, UgetNode* dnode)
 		// get real sibling
 		if (sibling)
 			sibling = sibling->real;
-		dnode->state = UGET_STATE_QUEUING;
+		relation->group = UGET_GROUP_QUEUING;
 		uget_node_insert (cnode, sibling, dnode);
 	}
 	return TRUE;
@@ -1086,21 +1129,19 @@ void  uget_app_reset_download_name (UgetApp* app, UgetNode* dnode)
 	UgetCommon*  common;
 	UgetNode*    sibling;
 	UgetNode*    cnode = NULL;
-	UgUri        uuri;
 
-	common = ug_info_realloc (&dnode->info, UgetCommonInfo);
+	common = ug_data_realloc(dnode->data, UgetCommonInfo);
 	if (common->file) {
-		if (dnode->name && strcmp (common->file, dnode->name) == 0)
+		if (common->name && strcmp(common->file, common->name) == 0)
 			return;
-		ug_free (dnode->name);
-		dnode->name = ug_strdup (common->file);
+		ug_free(common->name);
+		common->name = ug_strdup(common->file);
 		cnode = dnode->parent;
 	}
 	else if (common->uri) {
-		if (dnode->name && strcmp (common->uri, dnode->name) == 0)
+		if (common->name && strcmp(common->uri, common->name) == 0)
 			return;
-		ug_uri_init (&uuri, common->uri);
-		uget_node_set_name_by_uri (dnode, &uuri);
+		common->name = uget_name_from_uri_str(common->uri);
 		cnode = dnode->parent;
 	}
 
@@ -1108,9 +1149,9 @@ void  uget_app_reset_download_name (UgetApp* app, UgetNode* dnode)
 	if (cnode) {
 //		cnode   = dnode->parent;
 		sibling = dnode->next;
-		uget_node_remove (cnode, dnode);
-		uget_node_unref_fake (dnode);
-		uget_node_insert (cnode, sibling, dnode);
+		uget_node_remove(cnode, dnode);
+		uget_node_clear_fake(dnode);
+		uget_node_insert(cnode, sibling, dnode);
 	}
 }
 
@@ -1138,7 +1179,7 @@ void  uget_app_clear_attachment (UgetApp* app)
 	hash = uget_uri_hash_new ();
 	// add attachment
 	for (dnode = app->mix.children->children;  dnode;  dnode = dnode->next) {
-		if ((http = ug_info_get (&dnode->data->info, UgetHttpInfo)) == NULL)
+		if ((http = ug_data_get (dnode->data, UgetHttpInfo)) == NULL)
 			continue;
 		if (http->cookie_file)
 			uget_uri_hash_add (hash, http->cookie_file);
@@ -1185,7 +1226,8 @@ void  uget_app_clear_plugins (UgetApp* app)
 	pend = app->plugins.at + app->plugins.length;
 	for (pair = app->plugins.at;  pair < pend;  pair++) {
 		if (pair->data) {
-			uget_plugin_set (pair->data, UGET_PLUGIN_INIT, (void*) FALSE);
+			uget_plugin_global_set(pair->data, UGET_PLUGIN_GLOBAL_INIT,
+			                       (void*) FALSE);
 			pair->data = NULL;
 		}
 	}
@@ -1198,9 +1240,9 @@ void  uget_app_add_plugin (UgetApp* app, const UgetPluginInfo* pinfo)
 
 	pair = ug_registry_find (&app->plugins, pinfo->name, NULL);
 	if (pair == NULL || pair->data == NULL)
-		uget_plugin_set (pinfo, UGET_PLUGIN_INIT, (void*) TRUE);
+		uget_plugin_global_set(pinfo, UGET_PLUGIN_GLOBAL_INIT, (void*) TRUE);
 	if (pair == NULL)
-		ug_registry_add (&app->plugins, (const UgDataInfo*)pinfo);
+		ug_registry_add (&app->plugins, (const UgTypeInfo*)pinfo);
 }
 
 void  uget_app_remove_plugin (UgetApp* app, const UgetPluginInfo* pinfo)
@@ -1209,7 +1251,7 @@ void  uget_app_remove_plugin (UgetApp* app, const UgetPluginInfo* pinfo)
 
 	pair = ug_registry_find (&app->plugins, pinfo->name, NULL);
 	if (pair && pair->data) {
-		uget_plugin_set (pair->data, UGET_PLUGIN_INIT, (void*) FALSE);
+		uget_plugin_global_set(pair->data, UGET_PLUGIN_GLOBAL_INIT, (void*) FALSE);
 		pair->data = NULL;
 	}
 }
@@ -1283,6 +1325,17 @@ UgetPluginInfo*  uget_app_match_plugin (UgetApp* app,
 // ----------------------------------------------------------------------------
 // save/load categories
 
+// convert old format to new
+static void remove_file_node(UgetNode* cnode)
+{
+	UgetNode*  dnode;
+
+	for (dnode = cnode->children;  dnode;  dnode = dnode->next) {
+		while (dnode->children)
+			uget_node_free(dnode->children);
+	}
+}
+
 int   uget_app_save_category (UgetApp* app, UgetNode* cnode, const char* filename, void* jsonfile)
 {
 	int  fd;
@@ -1367,10 +1420,12 @@ UgetNode* uget_app_load_category_fd (UgetApp* app, int fd, void* jsonfile)
 		uget_node_make_fake (cnode);
 		// move all downloads from active to queuing in this category
 		uget_app_stop_category (app, cnode);
+		// convert old format to new
+		remove_file_node(cnode);
 		return cnode;
 	}
 	else {
-		uget_node_unref (cnode);
+		uget_node_free (cnode);
 		return NULL;
 	}
 }
@@ -1477,7 +1532,7 @@ void  uget_node_set_keeping (UgetNode* node, int enable)
 		UgetFtp*     ftp;
 	} temp;
 
-	temp.common = ug_info_realloc (&node->info, UgetCommonInfo);
+	temp.common = ug_data_realloc (node->data, UgetCommonInfo);
 	if (temp.common) {
 		temp.common->keeping.enable = enable;
 		if (enable) {
@@ -1515,7 +1570,7 @@ void  uget_node_set_keeping (UgetNode* node, int enable)
 		}
 	}
 
-	temp.proxy = ug_info_get (&node->info, UgetProxyInfo);
+	temp.proxy = ug_data_get (node->data, UgetProxyInfo);
 	if (temp.proxy) {
 		temp.proxy->keeping.enable = enable;
 		if (enable) {
@@ -1532,7 +1587,7 @@ void  uget_node_set_keeping (UgetNode* node, int enable)
 		}
 	}
 
-	temp.http = ug_info_get (&node->info, UgetHttpInfo);
+	temp.http = ug_data_get (node->data, UgetHttpInfo);
 	if (temp.http) {
 		temp.http->keeping.enable = enable;
 		if (enable) {
@@ -1557,7 +1612,7 @@ void  uget_node_set_keeping (UgetNode* node, int enable)
 		}
 	}
 
-	temp.ftp = ug_info_get (&node->info, UgetFtpInfo);
+	temp.ftp = ug_data_get (node->data, UgetFtpInfo);
 	if (temp.ftp) {
 		temp.ftp->keeping.enable = enable;
 		if (enable) {
