@@ -75,30 +75,37 @@ typedef struct  UgetApp          UgetApp;
 struct UgetApp
 {
 	UGET_APP_MEMBERS;
-//	UgetNode        real;           // real root node for real nodes
-//	UgetNode        split;          // virtual root
-//	UgetNode        sorted;         // virtual root
-//	UgetNode        sorted_split;   // virtual root
-//	UgetNode        mix;            // virtual root
-//	UgetNode        mix_split;      // virtual root
-//	UgRegistry      infos;
-//	UgRegistry      plugins;
-//	UgetPluginInfo* plugin;
-//	UgetTask        task;
-//	UgArrayPtr      nodes;
-//	void*           uri_hash;
-//	char*           config_dir;
-//	int             n_error;     // these n_xxxx increase by uget_app_grow()
-//	int             n_moved;
-//	int             n_deleted;
-//	int             n_completed;
+/*	// ------ UgetApp members ------
+	UgetNode        real;           // real root node for real nodes
+	UgetNode        split;          // virtual root
+	UgetNode        sorted;         // virtual root
+	UgetNode        sorted_split;   // virtual root
+	UgetNode        mix;            // virtual root
+	UgetNode        mix_split;      // virtual root
+	UgRegistry      infos;
+	UgRegistry      plugins;
+	UgetPluginInfo* plugin_default;
+	UgetTask        task;
+	UgArrayPtr      nodes;
+	void*           uri_hash;
+	char*           config_dir;
+	int             n_error;        // uget_app_grow() will count these value:
+	int             n_moved;        // n_error, n_moved, n_deleted, and
+	int             n_deleted;      // n_completed
+	int             n_completed;    //
+ */
 };
 
 void  uget_app_init (UgetApp* app);
 void  uget_app_final (UgetApp* app);
 
-// uget_app_grow() return number of active download
+// uget_app_grow() activate queue download
+//                 return number of active download
 int   uget_app_grow (UgetApp* app, int no_queuing);
+// uget_app_trim() remove finished/recycled download that over capacity
+//                 return number of trimmed download
+int   uget_app_trim (UgetApp* app);
+
 void  uget_app_set_config_dir (UgetApp* app, const char* dir);
 void  uget_app_set_sorting (UgetApp* app, UgCompareFunc func, int reversed);
 void  uget_app_set_notification (UgetApp* app, void* data,
