@@ -38,13 +38,16 @@
 #define UGET_DATA_H
 
 #include <stdint.h>
-#include <UgData.h>
+#include <UgUri.h>
+#include <UgGroupData.h>
+#include <UgetFiles.h>
 #include <UgetPlugin.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+// group data
 typedef struct  UgetCommon      UgetCommon;
 typedef struct  UgetProgress    UgetProgress;
 typedef struct  UgetProxy       UgetProxy;
@@ -54,24 +57,31 @@ typedef struct  UgetLog         UgetLog;
 typedef struct  UgetRelation    UgetRelation;
 typedef struct  UgetCategory    UgetCategory;
 
-extern const UgDataInfo*  UgetCommonInfo;
-extern const UgDataInfo*  UgetProgressInfo;
-extern const UgDataInfo*  UgetProxyInfo;
-extern const UgDataInfo*  UgetHttpInfo;
-extern const UgDataInfo*  UgetFtpInfo;
-extern const UgDataInfo*  UgetLogInfo;
-extern const UgDataInfo*  UgetRelationInfo;
-extern const UgDataInfo*  UgetCategoryInfo;
+extern const UgGroupDataInfo*   UgetCommonInfo;
+extern const UgGroupDataInfo*   UgetProgressInfo;
+extern const UgGroupDataInfo*   UgetProxyInfo;
+extern const UgGroupDataInfo*   UgetHttpInfo;
+extern const UgGroupDataInfo*   UgetFtpInfo;
+extern const UgGroupDataInfo*   UgetLogInfo;
+extern const UgGroupDataInfo*   UgetRelationInfo;
+extern const UgGroupDataInfo*   UgetCategoryInfo;
 
-// ----------------------------------------------------------------------------
-// UgetCommon: It derived from UgData and store in UgInfo.
+/* ----------------------------------------------------------------------------
+   UgetCommon: It derived from UgGroupData and store in UgData.
+
+   UgType
+   |
+   `-- UgGroupData
+       |
+       `-- UgetCommon
+ */
 
 struct UgetCommon
 {
-	UG_DATA_MEMBERS;           // It derived from UgData
-//	const UgDataInfo*  info;
+	UG_GROUP_DATA_MEMBERS;
+//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
 
-//	char*   name;
+	char*   name;
 	char*   uri;
 	char*   mirrors;
 	char*   file;
@@ -96,12 +106,12 @@ struct UgetCommon
 	// debug
 	int           debug_level;
 
-	// keeping flags used by ug_data_assign ()
+	// keeping flags used by ug_group_data_assign ()
 	// They works like read-only
 	struct {
 		uint8_t   enable:1;
 
-//		uint8_t   name:1;
+		uint8_t   name:1;
 		uint8_t   uri:1;
 		uint8_t   mirrors:1;
 		uint8_t   file:1;
@@ -123,13 +133,24 @@ struct UgetCommon
 	} keeping;
 };
 
-// ----------------------------------------------------------------------------
-// UgetProgress: It derived from UgData and store in UgInfo.
+// helper functions for UgetCommon::name
+char* uget_name_from_uri(UgUri* uri);
+char* uget_name_from_uri_str(const char* uri);
+
+/* ----------------------------------------------------------------------------
+   UgetProgress: It derived from UgGroupData and store in UgData.
+
+   UgType
+   |
+   `-- UgGroupData
+       |
+       `-- UgetProgress
+ */
 
 struct UgetProgress
 {
-	UG_DATA_MEMBERS;           // It derived from UgData
-//	const UgDataInfo*  info;
+	UG_GROUP_DATA_MEMBERS;
+//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
 
 	int64_t      elapsed;     // consume time (seconds)
 	int64_t      left;        // remain time  (seconds)
@@ -144,8 +165,15 @@ struct UgetProgress
 	int          percent;
 };
 
-// ----------------------------------------------------------------------------
-// UgetProxy: It derived from UgData and store in UgInfo.
+/* ----------------------------------------------------------------------------
+   UgetProxy: It derived from UgGroupData and store in UgData.
+
+   UgType
+   |
+   `-- UgGroupData
+       |
+       `-- UgetProxy
+ */
 
 typedef enum
 {
@@ -163,8 +191,8 @@ typedef enum
 
 struct UgetProxy
 {
-	UG_DATA_MEMBERS;           // It derived from UgData
-//	const UgDataInfo*  info;
+	UG_GROUP_DATA_MEMBERS;
+//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
 
 	char*          host;
 	unsigned int   port;
@@ -201,13 +229,20 @@ struct UgetProxy
 #endif	// HAVE_LIBPWMD
 };
 
-// ----------------------------------------------------------------------------
-// UgetHttp: It derived from UgData and store in UgInfo.
+/* ----------------------------------------------------------------------------
+   UgetHttp: It derived from UgGroupData and store in UgData.
+
+   UgType
+   |
+   `-- UgGroupData
+       |
+       `-- UgetHttp
+ */
 
 struct UgetHttp
 {
-	UG_DATA_MEMBERS;           // It derived from UgData
-//	const UgDataInfo*  info;
+	UG_GROUP_DATA_MEMBERS;
+//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
 
 	char*  user;
 	char*  password;
@@ -238,13 +273,20 @@ struct UgetHttp
 	} keeping;
 };
 
-// ----------------------------------------------------------------------------
-// UgetFtp: It derived from UgData and store in UgInfo.
+/* ----------------------------------------------------------------------------
+   UgetFtp: It derived from UgGroupData and store in UgData.
+
+   UgType
+   |
+   `-- UgGroupData
+       |
+       `-- UgetFtp
+ */
 
 struct UgetFtp
 {
-	UG_DATA_MEMBERS;           // It derived from UgData
-//	const UgDataInfo*  iface;
+	UG_GROUP_DATA_MEMBERS;
+//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
 
 	char*		 user;
 	char*		 password;
@@ -260,13 +302,20 @@ struct UgetFtp
 	} keeping;
 };
 
-// ---------------------------------------------------------------------------
-// UgetLog
+/* ---------------------------------------------------------------------------
+   UgetLog
+
+   UgType
+   |
+   `-- UgGroupData
+       |
+       `-- UgetLog
+ */
 
 struct UgetLog
 {
-	UG_DATA_MEMBERS;           // It derived from UgData
-//	const UgDataInfo*  info;
+	UG_GROUP_DATA_MEMBERS;
+//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
 
 	time_t  added_time;
 	time_t  completed_time;
@@ -274,8 +323,15 @@ struct UgetLog
 	UgList  messages;          // List for UgetEvent
 };
 
-// ----------------------------------------------------------------------------
-// UgetRelation: It derived from UgData and store in UgInfo.
+/* ----------------------------------------------------------------------------
+   UgetRelation: It derived from UgGroupData and store in UgData.
+
+   UgType
+   |
+   `-- UgGroupData
+       |
+       `-- UgetRelation
+ */
 
 typedef enum
 {
@@ -286,8 +342,10 @@ typedef enum
 
 struct UgetRelation
 {
-	UG_DATA_MEMBERS;           // It derived from UgData
-//	const UgDataInfo*  info;
+	UG_GROUP_DATA_MEMBERS;
+//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
+
+	int   group;     // UgetGroup
 
 	// used by UgetTask
 	struct UgetRelationTask {
@@ -315,13 +373,20 @@ struct UgetRelation
 	} user;
 };
 
-// ----------------------------------------------------------------------------
-// UgetCategory: It derived from UgData and store in UgInfo.
+/* ----------------------------------------------------------------------------
+   UgetCategory: It derived from UgGroupData and store in UgData.
+
+   UgType
+   |
+   `-- UgGroupData
+       |
+       `-- UgetCategory
+ */
 
 struct UgetCategory
 {
-	UG_DATA_MEMBERS;           // It derived from UgData
-//	const UgDataInfo*  info;
+	UG_GROUP_DATA_MEMBERS;
+//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
 
 	// use these to classify download
 	UgArrayStr    hosts;
@@ -330,10 +395,10 @@ struct UgetCategory
 
 	// limit
 	int        active_limit;
-	int        finished_limit; // finished: completed and paused
+	int        finished_limit;   // finished: completed and stopped
 	int        recycled_limit;
 
-	// 4 fake sub-category tree
+	// subcategory in UgetNode::fake
 	UgetNode*  active;
 	UgetNode*  queuing;
 	UgetNode*  finished;
@@ -354,14 +419,14 @@ namespace Uget
 {
 
 // These are for directly use only. You can NOT derived it.
-struct Common : Ug::DataMethod, UgetCommon {};
-struct Progress : Ug::DataMethod, UgetProgress {};
-struct Proxy : Ug::DataMethod, UgetProxy {};
-struct Http : Ug::DataMethod, UgetHttp {};
-struct Ftp : Ug::DataMethod, UgetFtp {};
-struct Log : Ug::DataMethod, UgetLog {};
-struct Relation : Ug::DataMethod, UgetRelation {};
-struct Category : Ug::DataMethod, UgetCategory {};
+struct Common : Ug::GroupDataMethod, UgetCommon {};
+struct Progress : Ug::GroupDataMethod, UgetProgress {};
+struct Proxy : Ug::GroupDataMethod, UgetProxy {};
+struct Http : Ug::GroupDataMethod, UgetHttp {};
+struct Ftp : Ug::GroupDataMethod, UgetFtp {};
+struct Log : Ug::GroupDataMethod, UgetLog {};
+struct Relation : Ug::GroupDataMethod, UgetRelation {};
+struct Category : Ug::GroupDataMethod, UgetCategory {};
 
 };  // namespace Uget
 
