@@ -63,11 +63,11 @@ typedef enum UgetMediaQuality
 {
 	UGET_MEDIA_QUALITY_UNKNOWN = -1,
 
-	UGET_MEDIA_QUALITY_240P,    // Youtube small
-	UGET_MEDIA_QUALITY_360P,    // Youtube medium
-	UGET_MEDIA_QUALITY_480P,    // Youtube large
-	UGET_MEDIA_QUALITY_720P,    // Youtube hd720
-	UGET_MEDIA_QUALITY_1080P,   // Youtube hd1080
+	UGET_MEDIA_QUALITY_240P,    // YouTube small
+	UGET_MEDIA_QUALITY_360P,    // YouTube medium
+	UGET_MEDIA_QUALITY_480P,    // YouTube large
+	UGET_MEDIA_QUALITY_720P,    // YouTube hd720
+	UGET_MEDIA_QUALITY_1080P,   // YouTube hd1080
 
 	UGET_MEDIA_N_QUALITY,
 } UgetMediaQuality;
@@ -81,13 +81,22 @@ typedef enum UgetMediaType
 	UGET_MEDIA_TYPE_3GPP,
 	UGET_MEDIA_TYPE_FLV,
 
+	// YouTube MIME type:
+	// audio/mp4;+codecs="mp4a.40.2"
+	// audio/webm;+codecs="vorbis"
+	// audio/webm;+codecs="opus"
+	UGET_MEDIA_AUDIO_MP4,
+	UGET_MEDIA_AUDIO_WEBM,
+//	UGET_MEDIA_AUDIO_WEBM_VORBIS,  // YouTube - audio/webm;+codecs="vorbis"
+//	UGET_MEDIA_AUDIO_WEBM_OPUS,    // YouTube - audio/webm;+codecs="opus"
+
 	UGET_MEDIA_N_TYPE,
 } UgetMediaType;
 
 
 struct UgetMedia
 {
-	UG_LIST_MEMBERS (UgetMediaItem);
+	UG_LIST_MEMBERS(UgetMediaItem);
 //	uintptr_t        size;
 //	UgetMediaItem*   head;
 //	UgetMediaItem*   tail;
@@ -110,35 +119,50 @@ struct UgetMedia
 	void*  data4;
 };
 
-UgetMedia*  uget_media_new (const char* url, UgetSiteId site_id);
-void        uget_media_free (UgetMedia* umedia);
-void        uget_media_clear (UgetMedia* umedia, int free_items);
+UgetMedia*  uget_media_new(const char* url, UgetSiteId site_id);
+void        uget_media_free(UgetMedia* umedia);
+void        uget_media_clear(UgetMedia* umedia, int free_items);
 
-int         uget_media_grab_items (UgetMedia* umedia, UgetProxy* proxy);
+int         uget_media_grab_items(UgetMedia* umedia, UgetProxy* proxy);
 
 // return begin of matched items. Don't free it
-UgetMediaItem*  uget_media_match (UgetMedia*          umedia,
-                                  UgetMediaMatchMode  mode,
-                                  UgetMediaQuality    quality,
-                                  UgetMediaType       type);
+UgetMediaItem*  uget_media_match(UgetMedia*          umedia,
+                                 UgetMediaMatchMode  mode,
+                                 UgetMediaQuality    quality,
+                                 UgetMediaType       type);
 
 // ----------------------------------------------------------------------------
 // UgetMediaItem
 
 struct UgetMediaItem
 {
-	UG_LINK_MEMBERS (UgetMediaItem, UgetMediaItem, self);
+	UG_LINK_MEMBERS(UgetMediaItem, UgetMediaItem, self);
 //	UgetMediaItem* self;
 //	UgetMediaItem* next;
 //	UgetMediaItem* prev;
 
 	char* url;
-	int   quality;    // 480p, 720p
+	int   quality;    // video - 480p, 720p
+	int   bitrate;    // audio
 	int   type;       // UgetMediaType
+
+	// for internal use only
+	int   order;
+	union {
+		int   integer;
+		char* string;
+		void* pointer;
+	} data;
+
+	union {
+		int   integer;
+		char* string;
+		void* pointer;
+	} data1;
 };
 
-UgetMediaItem*  uget_media_item_new (UgetMedia* umedia);
-void            uget_media_item_free (UgetMediaItem* umitem);
+UgetMediaItem*  uget_media_item_new(UgetMedia* umedia);
+void            uget_media_item_free(UgetMediaItem* umitem);
 
 #ifdef __cplusplus
 }
