@@ -44,20 +44,23 @@
 
 int   uget_node_compare_name (UgetNode* node1, UgetNode* node2)
 {
-	node1 = node1->data;
-	node2 = node2->data;
+	UgetCommon* common1;
+	UgetCommon* common2;
 
-	if (node1->name) {
-		if (node2->name == NULL)
+	common1 = ug_data_get(node1->data, UgetCommonInfo);
+	common2 = ug_data_get(node2->data, UgetCommonInfo);
+
+	if (common1 && common1->name) {
+		if (common2->name == NULL)
 			return 1;
 	}
 	else {
-		if (node2->name == NULL)
-			return 0;
-		else
+		if (common2 && common2->name)
 			return -1;
+		else
+			return 0;
 	}
-	return strcmp (node1->name, node2->name);
+	return strcmp (common1->name, common2->name);
 }
 
 int   uget_node_compare_complete (UgetNode* node1, UgetNode* node2)
@@ -65,10 +68,10 @@ int   uget_node_compare_complete (UgetNode* node1, UgetNode* node2)
 	UgetProgress*  progress1;
 	UgetProgress*  progress2;
 
-	node1 = node1->data;
-	node2 = node2->data;
-	progress1 = ug_info_get (&node1->info, UgetProgressInfo);
-	progress2 = ug_info_get (&node2->info, UgetProgressInfo);
+	node1 = node1->base;
+	node2 = node2->base;
+	progress1 = ug_data_get (node1->data, UgetProgressInfo);
+	progress2 = ug_data_get (node2->data, UgetProgressInfo);
 	if (progress1) {
 		if (progress2 == NULL)
 			return 1;
@@ -93,10 +96,10 @@ int   uget_node_compare_size (UgetNode* node1, UgetNode* node2)
 	UgetProgress*  progress1;
 	UgetProgress*  progress2;
 
-	node1 = node1->data;
-	node2 = node2->data;
-	progress1 = ug_info_get (&node1->info, UgetProgressInfo);
-	progress2 = ug_info_get (&node2->info, UgetProgressInfo);
+	node1 = node1->base;
+	node2 = node2->base;
+	progress1 = ug_data_get (node1->data, UgetProgressInfo);
+	progress2 = ug_data_get (node2->data, UgetProgressInfo);
 	if (progress1) {
 		if (progress2 == NULL)
 			return 1;
@@ -121,10 +124,10 @@ int   uget_node_compare_percent (UgetNode* node1, UgetNode* node2)
 	UgetProgress*  progress1;
 	UgetProgress*  progress2;
 
-	node1 = node1->data;
-	node2 = node2->data;
-	progress1 = ug_info_get (&node1->info, UgetProgressInfo);
-	progress2 = ug_info_get (&node2->info, UgetProgressInfo);
+	node1 = node1->base;
+	node2 = node2->base;
+	progress1 = ug_data_get (node1->data, UgetProgressInfo);
+	progress2 = ug_data_get (node2->data, UgetProgressInfo);
 	if (progress1) {
 		if (progress2 == NULL)
 			return 1;
@@ -149,10 +152,10 @@ int   uget_node_compare_elapsed (UgetNode* node1, UgetNode* node2)
 	UgetProgress*  progress1;
 	UgetProgress*  progress2;
 
-	node1 = node1->data;
-	node2 = node2->data;
-	progress1 = ug_info_get (&node1->info, UgetProgressInfo);
-	progress2 = ug_info_get (&node2->info, UgetProgressInfo);
+	node1 = node1->base;
+	node2 = node2->base;
+	progress1 = ug_data_get (node1->data, UgetProgressInfo);
+	progress2 = ug_data_get (node2->data, UgetProgressInfo);
 	if (progress1) {
 		if (progress2 == NULL)
 			return 1;
@@ -177,10 +180,10 @@ int   uget_node_compare_left (UgetNode* node1, UgetNode* node2)
 	UgetProgress*  progress1;
 	UgetProgress*  progress2;
 
-	node1 = node1->data;
-	node2 = node2->data;
-	progress1 = ug_info_get (&node1->info, UgetProgressInfo);
-	progress2 = ug_info_get (&node2->info, UgetProgressInfo);
+	node1 = node1->base;
+	node2 = node2->base;
+	progress1 = ug_data_get (node1->data, UgetProgressInfo);
+	progress2 = ug_data_get (node2->data, UgetProgressInfo);
 	if (progress1) {
 		if (progress2 == NULL)
 			return 1;
@@ -205,10 +208,10 @@ int   uget_node_compare_speed (UgetNode* node1, UgetNode* node2)
 	UgetProgress*  progress1;
 	UgetProgress*  progress2;
 
-	node1 = node1->data;
-	node2 = node2->data;
-	progress1 = ug_info_get (&node1->info, UgetProgressInfo);
-	progress2 = ug_info_get (&node2->info, UgetProgressInfo);
+	node1 = node1->base;
+	node2 = node2->base;
+	progress1 = ug_data_get (node1->data, UgetProgressInfo);
+	progress2 = ug_data_get (node2->data, UgetProgressInfo);
 	if (progress1) {
 		if (progress2 == NULL)
 			return 1;
@@ -233,10 +236,10 @@ int   uget_node_compare_upload_speed (UgetNode* node1, UgetNode* node2)
 	UgetProgress*  progress1;
 	UgetProgress*  progress2;
 
-	node1 = node1->data;
-	node2 = node2->data;
-	progress1 = ug_info_get (&node1->info, UgetProgressInfo);
-	progress2 = ug_info_get (&node2->info, UgetProgressInfo);
+	node1 = node1->base;
+	node2 = node2->base;
+	progress1 = ug_data_get (node1->data, UgetProgressInfo);
+	progress2 = ug_data_get (node2->data, UgetProgressInfo);
 	if (progress1) {
 		if (progress2 == NULL)
 			return 1;
@@ -261,10 +264,10 @@ int   uget_node_compare_uploaded (UgetNode* node1, UgetNode* node2)
 	UgetProgress*  progress1;
 	UgetProgress*  progress2;
 
-	node1 = node1->data;
-	node2 = node2->data;
-	progress1 = ug_info_get (&node1->info, UgetProgressInfo);
-	progress2 = ug_info_get (&node2->info, UgetProgressInfo);
+	node1 = node1->base;
+	node2 = node2->base;
+	progress1 = ug_data_get (node1->data, UgetProgressInfo);
+	progress2 = ug_data_get (node2->data, UgetProgressInfo);
 	if (progress1) {
 		if (progress2 == NULL)
 			return 1;
@@ -289,10 +292,10 @@ int   uget_node_compare_ratio (UgetNode* node1, UgetNode* node2)
 	UgetProgress*  progress1;
 	UgetProgress*  progress2;
 
-	node1 = node1->data;
-	node2 = node2->data;
-	progress1 = ug_info_get (&node1->info, UgetProgressInfo);
-	progress2 = ug_info_get (&node2->info, UgetProgressInfo);
+	node1 = node1->base;
+	node2 = node2->base;
+	progress1 = ug_data_get (node1->data, UgetProgressInfo);
+	progress2 = ug_data_get (node2->data, UgetProgressInfo);
 	if (progress1) {
 		if (progress2 == NULL)
 			return 1;
@@ -317,10 +320,10 @@ int   uget_node_compare_retry (UgetNode* node1, UgetNode* node2)
 	UgetCommon*  common1;
 	UgetCommon*  common2;
 
-	node1 = node1->data;
-	node2 = node2->data;
-	common1 = ug_info_get (&node1->info, UgetCommonInfo);
-	common2 = ug_info_get (&node2->info, UgetCommonInfo);
+	node1 = node1->base;
+	node2 = node2->base;
+	common1 = ug_data_get (node1->data, UgetCommonInfo);
+	common2 = ug_data_get (node2->data, UgetCommonInfo);
 	if (common1) {
 		if (common2 == NULL)
 			return 1;
@@ -342,20 +345,7 @@ int   uget_node_compare_retry (UgetNode* node1, UgetNode* node2)
 
 int   uget_node_compare_parent_name (UgetNode* node1, UgetNode* node2)
 {
-	node1 = node1->data->parent;
-	node2 = node2->data->parent;
-
-	if (node1->name) {
-		if (node2->name == NULL)
-			return 1;
-	}
-	else {
-		if (node2->name == NULL)
-			return 0;
-		else
-			return -1;
-	}
-	return strcmp (node1->name, node2->name);
+	return uget_node_compare_name(node1->base->parent, node2->base->parent);
 }
 
 int   uget_node_compare_uri (UgetNode* node1, UgetNode* node2)
@@ -363,10 +353,10 @@ int   uget_node_compare_uri (UgetNode* node1, UgetNode* node2)
 	UgetCommon*  common1;
 	UgetCommon*  common2;
 
-	node1 = node1->data;
-	node2 = node2->data;
-	common1 = ug_info_get (&node1->info, UgetCommonInfo);
-	common2 = ug_info_get (&node2->info, UgetCommonInfo);
+	node1 = node1->base;
+	node2 = node2->base;
+	common1 = ug_data_get (node1->data, UgetCommonInfo);
+	common2 = ug_data_get (node2->data, UgetCommonInfo);
 	if (common1) {
 		if (common2 == NULL)
 			return 1;
@@ -385,10 +375,10 @@ int   uget_node_compare_added_time (UgetNode* node1, UgetNode* node2)
 	UgetLog*  log1;
 	UgetLog*  log2;
 
-	node1 = node1->data;
-	node2 = node2->data;
-	log1 = ug_info_get (&node1->info, UgetLogInfo);
-	log2 = ug_info_get (&node2->info, UgetLogInfo);
+	node1 = node1->base;
+	node2 = node2->base;
+	log1 = ug_data_get (node1->data, UgetLogInfo);
+	log2 = ug_data_get (node2->data, UgetLogInfo);
 	if (log1) {
 		if (log2 == NULL)
 			return 1;
@@ -413,10 +403,10 @@ int   uget_node_compare_completed_time (UgetNode* node1, UgetNode* node2)
 	UgetLog*  log1;
 	UgetLog*  log2;
 
-	node1 = node1->data;
-	node2 = node2->data;
-	log1 = ug_info_get (&node1->info, UgetLogInfo);
-	log2 = ug_info_get (&node2->info, UgetLogInfo);
+	node1 = node1->base;
+	node2 = node2->base;
+	log1 = ug_data_get (node1->data, UgetLogInfo);
+	log2 = ug_data_get (node2->data, UgetLogInfo);
 	if (log1) {
 		if (log2 == NULL)
 			return 1;
