@@ -38,12 +38,6 @@
 #include <config.h>
 #endif
 
-#ifdef _MSC_VER
-#define _CRT_SECURE_NO_WARNINGS
-#endif
-
-#include <stdint.h>
-#include <UgUtil.h>
 #include <UgString.h>
 #include <UgetNode.h>
 #include <UgetData.h>
@@ -52,23 +46,38 @@
 #include <glib.h>    // g_slice_xxx
 #endif
 
+static void  uget_node_call_fake_filter (UgetNode* parent, UgetNode* sibling, UgetNode* child);
+static UgJsonError  ug_json_parse_state2group (UgJson* json,
+                                const char* name, const char* value,
+                                void* node, void* none);
+static UgJsonError  ug_json_parse_name2data (UgJson* json,
+                                const char* name, const char* value,
+                                void* node, void* none);
 // ----------------------------------------------------------------------------
 // UgetNode
 
-static struct UgetNodeNotification  notification = {
-	(UgetNodeFunc) NULL, NULL, NULL,
-	(UgNotifyFunc) NULL, NULL, 0, NULL
+static struct UgetNodeControl  control = {
+	NULL,                   // struct UgetNodeControl*  children;
+	{NULL, NULL, NULL},     // struct UgetNodeNotifier  notifier;
+	{NULL, FALSE},          // struct UgetNodeSort      sort;
+	NULL,                   // UgetNodeFunc             filter;
+	NULL                    // void*                    data;
 };
 
 const UgEntry  UgetNodeEntry[] =
 {
-	{"name",     offsetof (UgetNode, name),  UG_ENTRY_STRING, NULL, NULL},
-	{"type",     offsetof (UgetNode, type),  UG_ENTRY_INT,    NULL, NULL},
-	{"state",    offsetof (UgetNode, state), UG_ENTRY_INT,    NULL, NULL},
-	{"info",     offsetof (UgetNode, info),  UG_ENTRY_CUSTOM,
-			ug_json_parse_info,   ug_json_write_info},
+	{"data",     offsetof (UgetNode, data),  UG_ENTRY_CUSTOM,
+			ug_json_parse_data_ptr,   ug_json_write_data_ptr},
 	{"children", 0,                          UG_ENTRY_ARRAY,
 			ug_json_parse_uget_node_children, ug_json_write_uget_node_children},
+
+	// deprecated
+	{"name",     0,                          UG_ENTRY_CUSTOM,
+			ug_json_parse_name2data,  NULL},
+	{"state",    0,                          UG_ENTRY_CUSTOM,
+			ug_json_parse_state2group, NULL},
+	{"info",     offsetof (UgetNode, data),  UG_ENTRY_CUSTOM,
+			ug_json_parse_data_ptr,   NULL},
 	{NULL}    // null-terminated
 };
 
@@ -89,77 +98,68 @@ void  uget_node_init  (UgetNode* node, UgetNode* node_real)
 {
 	memset (node, 0, sizeof (UgetNode));
 
-	node->ref_count = 1;
-	node->notification = &notification;
+	node->control = &control;
 
 	if (node_real == NULL) {
-		node->data = node;    // pointer to self
-		ug_info_init (&node->info, 8, 2);
-		node->info.at[0].key = (void*) UgetRelationInfo;
-		node->info.at[1].key = (void*) UgetProgressInfo;
+		node->base = node;    // pointer to self
+		node->data = ug_data_new(8, 2);
+		node->data->at[0].key = (void*) UgetRelationInfo;
+		node->data->at[1].key = (void*) UgetProgressInfo;
 	}
 	else {
 		// this is a fake node.
-//		node->state = 0;
-		node->data = node_real->data;
-		node->type = node_real->type;
+//		node->group = 0;
+		node->base = node_real->base;
 		node->real = node_real;
 		node->peer = node_real->fake;
 		node_real->fake = node;
-		ug_info_init (&node->info, 0, 0);
+		node->data = node_real->data;
+		ug_data_ref(node->data);
 	}
 }
 
-void  uget_node_ref (UgetNode* node)
+void  uget_node_free (UgetNode* node)
 {
-	node->ref_count++;
-}
+	if (node->parent)
+		uget_node_remove (node->parent, node);
+	if (node->real)
+		uget_node_remove_fake (node->real, node);
 
-void  uget_node_unref (UgetNode* node)
-{
-	if (--node->ref_count == 0) {
-		if (node->parent)
-			uget_node_remove (node->parent, node);
-		if (node->real)
-			uget_node_remove_fake (node->real, node);
-
-		uget_node_unref_fake (node);
-		uget_node_unref_children (node);
-//		ug_node_unlink ((UgNode*)node);
-		ug_info_final (&node->info);
-		ug_free (node->name);
+	uget_node_clear_fake (node);
+	uget_node_clear_children (node);
+//	ug_node_unlink ((UgNode*)node);
+	ug_data_unref(node->data);
 
 #ifdef HAVE_GLIB
-		g_slice_free1 (sizeof (UgetNode), node);
+	g_slice_free1 (sizeof (UgetNode), node);
 #else
-		ug_free (node);
+	ug_free (node);
 #endif // HAVE_GLIB
-	}
 }
 
-void  uget_node_unref_children (UgetNode* node)
+void  uget_node_clear_children (UgetNode* node)
 {
 	UgetNode*  next;
 	UgetNode*  children;
 
 	for (children = node->children;  children;  children = next) {
 		next = children->next;
-		uget_node_unref (children);
+		uget_node_free(children);
 	}
 	node->children = NULL;
 }
 
-void  uget_node_unref_fake (UgetNode* node)
+void  uget_node_clear_fake (UgetNode* node)
 {
 	UgetNode*  peer;
 	UgetNode*  fake;
 
 	for (fake = node->fake;  fake;  fake = peer) {
 		peer = fake->peer;
-		fake->real = NULL;   // speed up uget_node_unref()
-		uget_node_unref (fake);
+		fake->real = NULL;   // speed up uget_node_free()
+		uget_node_free(fake);
 	}
-	node->fake = NULL;       // speed up uget_node_unref()
+	node->fake = NULL;       // speed up uget_node_free()
 }
 
 void  uget_node_move (UgetNode* node, UgetNode* sibling, UgetNode* child)
@@ -194,13 +194,13 @@ void  uget_node_insert (UgetNode* node, UgetNode* sibling, UgetNode* child)
 	UgetNodeFunc inserted;
 
 	ug_node_insert ((UgNode*) node, (UgNode*) sibling, (UgNode*) child);
-	child->notification = node->notification;
+	child->control = node->control;
 
-	inserted = node->notification->inserted;
+	inserted = node->control->notifier.inserted;
 	if (inserted)
 		inserted (node, sibling, child);
 
-	uget_node_created (node, sibling, child);
+	uget_node_call_fake_filter (node, sibling, child);
 }
 
 static void  uget_node_unlink_children_real (UgetNode* node)
@@ -226,7 +226,7 @@ static void  uget_node_unlink_fake_parent (UgetNode* child)
 		sibling = child->next;
 		ug_node_remove ((UgNode*) parent, (UgNode*) child);
 		// notify
-		removed = parent->notification->removed;
+		removed = parent->control->notifier.removed;
 		if (removed)
 			removed (parent, sibling, child);
 	}
@@ -242,7 +242,7 @@ void  uget_node_remove (UgetNode* node, UgetNode* child)
 	uget_node_unlink_fake_parent (child);
 	uget_node_unlink_children_real (child);
 
-	removed = node->notification->removed;
+	removed = node->control->notifier.removed;
 	if (removed)
 		removed (node, sibling, child);
 }
@@ -252,13 +252,13 @@ void  uget_node_append (UgetNode* node, UgetNode* child)
 	UgetNodeFunc inserted;
 
 	ug_node_append ((UgNode*) node, (UgNode*) child);
-	child->notification = node->notification;
+	child->control = node->control;
 
-	inserted = node->notification->inserted;
+	inserted = node->control->notifier.inserted;
 	if (inserted)
 		inserted (node, NULL, child);
 
-	uget_node_created (node, NULL, child);
+	uget_node_call_fake_filter (node, NULL, child);
 }
 
 void  uget_node_prepend (UgetNode* node, UgetNode* child)
@@ -268,13 +268,13 @@ void  uget_node_prepend (UgetNode* node, UgetNode* child)
 
 	sibling = node->children;
 	ug_node_prepend ((UgNode*) node, (UgNode*) child);
-	child->notification = node->notification;
+	child->control = node->control;
 
-	inserted = node->notification->inserted;
+	inserted = node->control->notifier.inserted;
 	if (inserted)
 		inserted (node, sibling, child);
 
-	uget_node_created (node, sibling, child);
+	uget_node_call_fake_filter (node, sibling, child);
 }
 
 void  uget_node_sort (UgetNode* node, UgCompareFunc compare, int reversed)
@@ -307,14 +307,14 @@ void  uget_node_insert_sorted (UgetNode* node, UgetNode* child)
 {
 	UgCompareFunc  compare;
 	UgetNode*      cur;
-	int            reversed;
+	int            reverse;
 
-	compare = node->notification->compare;
-	reversed = node->notification->reversed;
+	compare = node->control->sort.compare;
+	reverse = node->control->sort.reverse;
 	if (compare == NULL)
 		return;
 
-	if (reversed == FALSE) {
+	if (reverse == FALSE) {
 		for (cur = node->children;  cur;  cur = cur->next) {
 			if (compare (cur, child) > 0) {
 				uget_node_insert (node, cur, child);
@@ -406,11 +406,25 @@ void  uget_node_make_fake (UgetNode* node)
 	if (node->fake == NULL)
 		return;
 	for (child = node->children;  child;  child = child->next) {
-		uget_node_created (node, NULL, child);
+		uget_node_call_fake_filter (node, NULL, child);
 		uget_node_make_fake (child);
 	}
 }
 
+// fake filter node from real.
+// If real node inserted a child node, all fake nodes call this to filter.
+static void  uget_node_call_fake_filter (UgetNode* parent, UgetNode* sibling, UgetNode* child)
+{
+	UgetNode*    fake;
+	UgetNodeFunc filter;
+
+	for (fake = parent->fake;  fake;  fake = fake->peer) {
+		filter = fake->control->filter;
+		if (filter)
+			filter (fake, sibling, child);
+	}
+}
+
 // ----------------------------------------------------------------------------
 // position
 UgetNode* uget_node_nth_fake (UgetNode* node, int nth)
@@ -424,15 +438,6 @@ UgetNode* uget_node_nth_fake (UgetNode* node, int nth)
 	return NULL;
 }
 
-UgetNode* uget_node_fake_from_state (UgetNode* node, int state)
-{
-	for (node = node->fake;  node;  node = node->peer) {
-		if (node->state & state)
-			return node;
-	}
-	return NULL;
-}
-
 int  uget_node_fake_position (UgetNode* node, UgetNode* fake)
 {
 	int  position = 0;
@@ -444,66 +449,14 @@ int  uget_node_fake_position (UgetNode* node, UgetNode* fake)
 	return -1;
 }
 
-// ----------------------------------------------------------------------------
-
-void  uget_node_set_name_by_uri (UgetNode* node, UgUri* uuri)
-{
-	const char* filename;
-	int         length;
-
-	ug_free (node->name);
-
-	if (uuri->scheme_len == 6 && strncmp (uuri->uri, "magnet", 6) == 0) {
-		length = 0;
-		filename = strstr (uuri->uri + uuri->file, "dn=");
-		if (filename) {
-			filename = filename + 3;
-			length = strcspn (filename, "&");
-			node->name = ug_malloc (length + 1);
-			ug_decode_uri (filename, length, node->name);
-			if (ug_utf8_get_invalid (node->name, NULL) != -1) {
-				ug_free (node->name);
-				node->name = ug_strndup (filename, length);
-			}
-		}
-	}
-	else {
-		length = ug_uri_file (uuri, &filename);
-		if (length == 0)
-			node->name = ug_strdup (uuri->uri);
-		else
-			node->name = ug_uri_get_file (uuri);
-	}
-}
-
-void  uget_node_set_name_by_uri_string (UgetNode* node, const char* uri)
-{
-	UgUri  uuri;
-
-	ug_uri_init (&uuri, uri);
-	uget_node_set_name_by_uri (node, &uuri);
-}
-
 // ----------------------------------------------------------------------------
 // notify
 
-void  uget_node_created (UgetNode* parent, UgetNode* sibling, UgetNode* child)
-{
-	UgetNode*    fake;
-	UgetNodeFunc created;
-
-	for (fake = parent->fake;  fake;  fake = fake->peer) {
-		created = fake->notification->created;
-		if (created)
-			created (fake, sibling, child);
-	}
-}
-
 void  uget_node_updated (UgetNode* node)
 {
 	UgNotifyFunc updated;
 
-	updated = node->notification->updated;
+	updated = node->control->notifier.updated;
 	if (updated)
 		updated (node);
 	// update fake node
@@ -541,167 +494,51 @@ void  ug_json_write_uget_node_children (UgJson* json, const UgetNode* node)
 	}
 }
 
-// ----------------------------------------------------------------------------
-// callback functions for UgetNode.notification.created
-// These functions used by UgetApp
-
-// sibling_real, child_real
-void  uget_node_create_split (UgetNode* node, UgetNode* sibling, UgetNode* child_real)
+// convert old format to new
+static UgJsonError  ug_json_parse_name2data (UgJson* json,
+                                const char* name, const char* value,
+                                void* nodev, void* none)
 {
-	UgetNode*  child;
+	UgetNode*     node = nodev;
+	union {
+		UgetCommon*   common;
+		UgetFiles*    files;
+	} temp;
 
-	if (node->parent == NULL) {
-		// node is root. child_real is category
-		//
-		child = uget_node_new (child_real);
-		child->state = UGET_STATE_RECYCLED;
-		uget_node_prepend (node, child);
-		//
-		child = uget_node_new (child_real);
-		child->state = UGET_STATE_FINISHED;
-		uget_node_prepend (node, child);
-		//
-		child = uget_node_new (child_real);
-		child->state = UGET_STATE_QUEUING;
-		uget_node_prepend (node, child);
-		//
-		child = uget_node_new (child_real);
-		child->state = UGET_STATE_ACTIVE;
-		uget_node_prepend (node, child);
-	}
-	else if (node->parent->parent == NULL) {
-		// node is category. child_real is download
-		child = child_real->data;
-		if ((node->state & child->state) ||
-		    ( (child->state & UGET_STATE_CATEGORY) == 0 &&
-		      (node->state & UGET_STATE_QUEUING) ) )
-		{
-			// insert sorted
-			if (node->notification->compare) {
-				uget_node_insert_sorted (node, uget_node_new (child_real));
-				return;
-			}
+	if (json->type != UG_JSON_STRING)
+		return UG_JSON_ERROR_TYPE_NOT_MATCH;
 
-			// original order
-			if (sibling) {
-				for (sibling = sibling->fake;  sibling;  sibling = sibling->peer) {
-					if (sibling->parent == node)
-						break;
-				}
-			}
-			uget_node_insert (node, sibling, uget_node_new (child_real));
-		}
+	// Now root node is category node
+	if (node->parent == NULL || node->parent->parent == NULL) {
+		// category or download node
+		temp.common = ug_data_realloc(node->data, UgetCommonInfo);
+		temp.common->name = ug_strdup(value);
 	}
+	else if (node->parent->parent->parent == NULL) {
+		// file node
+		temp.files = ug_data_realloc(node->parent->data, UgetFilesInfo);
+		uget_files_realloc(temp.files, value);
+	}
+	return UG_JSON_ERROR_NONE;
 }
 
-void  uget_node_create_sorted (UgetNode* node, UgetNode* sibling, UgetNode* child)
+// convert old format to new
+static UgJsonError  ug_json_parse_state2group (UgJson* json,
+                                const char* name, const char* value,
+                                void* nodev, void* none)
 {
-	child = uget_node_new (child);
+	UgetRelation* relation;
+	UgetNode*     node = nodev;
+	int           group;
 
-	if (node->parent == NULL) {
-		// node is root.
-		uget_node_append (node, child);
-	}
-	else if (node->parent->parent == NULL) {
-		// node is category.
-		// insert sorted
-		if (node->notification->compare) {
-			uget_node_insert_sorted (node, child);
-			return;
+	if (json->type != UG_JSON_NUMBER)
+		return UG_JSON_ERROR_TYPE_NOT_MATCH;
+	else {
+		group = strtol(value, NULL, 10);
+		if (group != 0) {
+			relation = ug_data_realloc(node->data, UgetRelationInfo);
+			relation->group = group;
 		}
-
-		// original order
-		if (sibling) {
-			for (sibling = sibling->fake;  sibling;  sibling = sibling->peer) {
-				if (sibling->parent == node)
-					break;
-			}
-		}
-
-		uget_node_insert (node, sibling, child);
 	}
-}
-
-// This one is not the same with uget_node_create_sorted()
-// sibling_real, child_real
-void  uget_node_create_mix (UgetNode* node, UgetNode* sibling, UgetNode* child)
-{
-	UgetNode*  fake;
-
-	child = uget_node_new (child);
-
-	if (node->parent == NULL) {
-		// node is root.
-		uget_node_append (node, child);
-	}
-	else if (node->parent->parent == NULL) {
-		// node is category.
-		// insert sorted
-		node = node->parent->children;
-		if (node->notification->compare) {
-			// add all download to first category
-			uget_node_insert_sorted (node, child);
-			return;
-		}
-
-		// reorder by state
-		if ( sibling &&
-		    ((sibling->data->state & UGET_STATE_CATEGORY) !=
-		     (child->data->state   & UGET_STATE_CATEGORY)) )
-		{
-			sibling = NULL;
-		}
-		if (node->fake && sibling == NULL) {
-			switch (child->data->state & UGET_STATE_CATEGORY) {
-			case UGET_STATE_ACTIVE:
-				fake = uget_node_fake_from_state (node, UGET_STATE_QUEUING);
-				if (fake == NULL || fake->children == NULL)
-					fake = uget_node_fake_from_state (node, UGET_STATE_FINISHED);
-				if (fake == NULL || fake->children == NULL)
-					fake = uget_node_fake_from_state (node, UGET_STATE_RECYCLED);
-				break;
-
-			case UGET_STATE_QUEUING:
-			default:
-				fake = uget_node_fake_from_state (node, UGET_STATE_FINISHED);
-				if (fake == NULL || fake->children == NULL)
-					fake = uget_node_fake_from_state (node, UGET_STATE_RECYCLED);
-				break;
-
-			case UGET_STATE_FINISHED:
-				fake = uget_node_fake_from_state (node, UGET_STATE_RECYCLED);
-				break;
-
-			case UGET_STATE_RECYCLED:
-				fake = NULL;
-				break;
-			}
-			if (fake && fake->children) {
-				uget_node_insert (node, fake->children->real, child);
-				return;
-			}
-		}
-
-		// original order
-		if (sibling) {
-			for (sibling = sibling->fake;  sibling;  sibling = sibling->peer) {
-//				if (sibling->parent == node)
-				if (sibling->parent == node)
-					break;
-			}
-		}
-		// insert childNode to first (mixed) category
-		uget_node_insert (node, sibling, child);
-	}
-}
-
-void  uget_node_create_mix_split (UgetNode* node, UgetNode* sibling, UgetNode* child_real)
-{
-	UgetNode*  real;
-
-	// if REAL category node is not first child, this function do nothing.
-	real = node->real;
-	if (real->parent == NULL && real->children != child_real)
-		return;
-	uget_node_create_split (node, sibling, child_real);
+	return UG_JSON_ERROR_NONE;
 }
