@@ -46,23 +46,22 @@
 extern "C" {
 #endif
 
-typedef struct Aria2File                 Aria2File;
 typedef struct UgetPluginAria2           UgetPluginAria2;
 typedef struct UgetPluginAria2Setting    UgetPluginAria2Setting;
 
 extern  const  UgetPluginInfo*           UgetPluginAria2Info;
 
 typedef enum {
-	UGET_PLUGIN_ARIA2_OPTION = UGET_PLUGIN_OPTION_DERIVED,
-	UGET_PLUGIN_ARIA2_URI,       // set parameter = (char* )
-//	UGET_PLUGIN_ARIA2_LOCAL,     // set parameter = (intptr_t)
-	UGET_PLUGIN_ARIA2_PATH,      // set parameter = (char* )
-	UGET_PLUGIN_ARIA2_ARGUMENT,  // set parameter = (char* )
-	UGET_PLUGIN_ARIA2_TOKEN,     // set parameter = (char* )
-	UGET_PLUGIN_ARIA2_LAUNCH,    // get/set parameter = (intptr_t)
-	UGET_PLUGIN_ARIA2_SHUTDOWN,  // set parameter = (intptr_t)
-	UGET_PLUGIN_ARIA2_SHUTDOWN_NOW,  // set parameter = (intptr_t)
-} UgetPluginAria2Code;
+	UGET_PLUGIN_ARIA2_GLOBAL = UGET_PLUGIN_GLOBAL_DERIVED,
+	UGET_PLUGIN_ARIA2_GLOBAL_URI,       // set parameter = (char* )
+//	UGET_PLUGIN_ARIA2_GLOBAL_LOCAL,     // set parameter = (intptr_t)
+	UGET_PLUGIN_ARIA2_GLOBAL_PATH,      // set parameter = (char* )
+	UGET_PLUGIN_ARIA2_GLOBAL_ARGUMENT,  // set parameter = (char* )
+	UGET_PLUGIN_ARIA2_GLOBAL_TOKEN,     // set parameter = (char* )
+	UGET_PLUGIN_ARIA2_GLOBAL_LAUNCH,    // get/set parameter = (intptr_t)
+	UGET_PLUGIN_ARIA2_GLOBAL_SHUTDOWN,  // set parameter = (intptr_t)
+	UGET_PLUGIN_ARIA2_GLOBAL_SHUTDOWN_NOW,  // set parameter = (intptr_t)
+} UgetPluginAria2GlobalCode;
 
 typedef enum {
 	UGET_PLUGIN_ARIA2_ERROR_NONE,
@@ -70,31 +69,27 @@ typedef enum {
 	UGET_PLUGIN_ARIA2_ERROR_LAUNCH,
 } UgetPluginAria2Error;
 
-// ----------------------------------------------------------------------------
-// UgetPluginAria2File
+/* ----------------------------------------------------------------------------
+   UgetPluginAria2: aria2 plug-in that derived from UgetPlugin.
 
-struct Aria2File
-{
-	char*    path;
-	int64_t  completedLength;
-	int64_t  length;
-};
-
-typedef UG_ARRAY(Aria2File)    Aria2FileArray;
-
-// ----------------------------------------------------------------------------
-// UgetPluginAria2: It derived from UgetPlugin.
+   UgType
+   |
+   `--- UgetPlugin
+        |
+        `--- UgetPluginAria2
+ */
 
 struct UgetPluginAria2
 {
-	UGET_PLUGIN_MEMBERS;               // It derived from UgetPlugin
-//	const UgetPluginInfo*  info;
-//	UgetEvent*    messages;
-//	UgMutex       mutex;
-//	int           ref_count;
+	UGET_PLUGIN_MEMBERS;
+/*	// ------ UgType members ------
+	const UgetPluginInfo*  info;
 
-	// pointer to UgetNode that store in UgetApp
-	UgetNode*     node;
+	// ------ UgetPlugin members ------
+	UgetEvent*    messages;
+	UgMutex       mutex;
+	int           ref_count;
+ */
 
 	// aria2.addUri, aria2.addTorrent, aria2.addMetalink
 	UgJsonrpcObject*  start_request;
@@ -104,9 +99,8 @@ struct UgetPluginAria2
 	unsigned int      retry_delay;
 	// all gids and it's files
 	UgArrayStr        gids;
-	Aria2FileArray    files;
+	UgetFiles*        files;
 	int               files_per_gid;
-	int               files_per_gid_prev;
 
 	// aria2.tellStatus
 	int        status;
@@ -120,16 +114,16 @@ struct UgetPluginAria2
 	// speed limit control
 	// limit[0] = download speed limit
 	// limit[1] = upload speed limit
-	int               limit[2];
-	uint8_t           limit_changed:1;
-	uint8_t           limit_by_user:1; // speed limit changed by user
+	int        limit[2];
+	int        limit_upper[2];
+	uint8_t    limit_changed:1;  // speed limit changed by user or program
 
 	// flags
 	uint8_t    synced:1;
 	uint8_t    paused:1;    // paused by user or program
 	uint8_t    stopped:1;   // download is stopped
 	uint8_t    restart:1;   // for retry
-	uint8_t    node_named:1;
+	uint8_t    named:1;
 };
 
 // ----------------------------------------------------------------------------
