@@ -48,32 +48,47 @@
 extern "C" {
 #endif
 
-typedef struct UgetPluginCurl     UgetPluginCurl;
+typedef struct UgetPluginCurl    UgetPluginCurl;
 
-extern const UgetPluginInfo*  UgetPluginCurlInfo;
+extern const   UgetPluginInfo*   UgetPluginCurlInfo;
 
-// ----------------------------------------------------------------------------
-// UgetPluginCurl: libcurl plug-in that derived from UgetPlugin.
+/* ----------------------------------------------------------------------------
+   UgetPluginCurl: libcurl plug-in that derived from UgetPlugin.
+
+   UgType
+   |
+   `---UgetPlugin
+       |
+       `--- UgetPluginCurl
+ */
 
 struct UgetPluginCurl
 {
-	UGET_PLUGIN_MEMBERS;           // It derived from UgetPlugin
-//	const UgetPluginInfo*  info;
-//	UgetEvent*    messages;
-//	UgMutex       mutex;
-//	int           ref_count;
+	UGET_PLUGIN_MEMBERS;
+/*	// ------ UgType members ------
+	const UgetPluginInfo*  info;
 
-	// pointer to UgetNode that store in UgetApp
-	UgetNode*     node;
+	// ------ UgetPlugin members ------
+	UgetEvent*    messages;
+	UgMutex       mutex;
+	int           ref_count;
+ */
 
-	// copy of UgetNode data
+	// copy these UgGroupData from UgData that store in UgetApp
 	UgetCommon*   common;
+	UgetFiles*    files;
 	UgetProxy*    proxy;
 	UgetHttp*     http;
 	UgetFtp*      ftp;
 
-	// run-time info
+	// run-time
 //	struct curl_slist*  ftp_command;
+
+	struct {
+		char*     path;        // folder
+		int       length;
+	} folder;
+
 	struct {
 		char*     path;        // folder + filename
 		time_t    time;        // date and time
@@ -82,7 +97,7 @@ struct UgetPluginCurl
 
 	// aria2 control file
 	struct {
-		char*     path;
+		char*     path;        // folder + filename + ".aria2"
 		UgetA2cf  ctrl;
 	} aria2;
 
@@ -117,10 +132,8 @@ struct UgetPluginCurl
 	} base, size, speed, limit;
 
 	// flags
-	uint8_t       limit_by_user:1; // speed limit changed by user
-	uint8_t       limit_changed:1; // speed limit changed
+	uint8_t       limit_changed:1; // speed limit changed by user or program
 	uint8_t       file_renamed:1;  // has file path?
-	uint8_t       a2cf_named:1;    // has aria2 ctrl file name?
 	uint8_t       synced:1;
 	uint8_t       paused:1;        // paused by user or program
 	uint8_t       stopped:1;       // all of downloading thread are stopped
