@@ -40,14 +40,15 @@
 #include <UgetPluginEmpty.h>
 
 // ----------------------------------------------------------------------------
-// UgetPluginInfo (derived from UgDataInfo)
+// UgetPluginInfo (derived from UgGroupDataInfo)
 
-static void plugin_init  (UgetPluginEmpty* plugin);
-static void plugin_final (UgetPluginEmpty* plugin);
-static int  plugin_ctrl  (UgetPluginEmpty* plugin, int code, void* data);
-static int  plugin_sync  (UgetPluginEmpty* plugin);
-static UgetResult  global_set (int code, void* parameter);
-static UgetResult  global_get (int code, void* parameter);
+static void plugin_init (UgetPluginEmpty* plugin);
+static void plugin_final(UgetPluginEmpty* plugin);
+static int  plugin_ctrl (UgetPluginEmpty* plugin, int code, void* data);
+static int  plugin_accept(UgetPluginEmpty* plugin, UgData* data);
+static int  plugin_sync  (UgetPluginEmpty* plugin, UgData* data);
+static UgetResult  global_set(int code, void* parameter);
+static UgetResult  global_get(int code, void* parameter);
 
 static const char* schemes[] = {"http", "ftp", NULL};
 static const char* types[]   = {"torrent", NULL};
@@ -55,18 +56,17 @@ static const char* types[]   = {"torrent", NULL};
 static const UgetPluginInfo UgetPluginEmptyInfoStatic =
 {
 	"empty",
-	sizeof (UgetPluginEmpty),
-	(const UgEntry*) NULL,
+	sizeof(UgetPluginEmpty),
 	(UgInitFunc)   plugin_init,
 	(UgFinalFunc)  plugin_final,
-	(UgAssignFunc) NULL,
-	(UgetPluginCtrlFunc) plugin_ctrl,
+	(UgetPluginSyncFunc) plugin_accept,
 	(UgetPluginSyncFunc) plugin_sync,
+	(UgetPluginCtrlFunc) plugin_ctrl,
 	NULL,
 	schemes,
 	types,
-	(UgetPluginSetFunc) global_set,
-	(UgetPluginGetFunc) global_get
+	(UgetPluginGlobalFunc) global_set,
+	(UgetPluginGlobalFunc) global_get
 };
 // extern
 const UgetPluginInfo* UgetPluginEmptyInfo = &UgetPluginEmptyInfoStatic;
@@ -80,7 +80,7 @@ static struct
 	int  ref_count;
 } global = {0, 0};
 
-static UgetResult  global_init (void)
+static UgetResult  global_init(void)
 {
 	if (global.initialized == FALSE) {
 		//
@@ -89,18 +89,18 @@ static UgetResult  global_init (void)
 		//     return UGET_RESULT_ERROR;
 		//
 		global.initialized = TRUE;
-		puts ("UgetPluginEmpty global initialize");
+		puts("UgetPluginEmpty global initialize");
 	}
 	global.ref_count++;
 	return UGET_RESULT_OK;
 }
 
-static void  global_ref (void)
+static void  global_ref(void)
 {
 	global.ref_count++;
 }
 
-static void  global_unref (void)
+static void  global_unref(void)
 {
 	if (global.initialized == FALSE)
 		return;
@@ -111,19 +111,19 @@ static void  global_unref (void)
 		//
 		// your global finalized code
 		//
-		puts ("UgetPluginEmpty global finalize");
+		puts("UgetPluginEmpty global finalize");
 	}
 }
 
-static UgetResult  global_set (int option, void* parameter)
+static UgetResult  global_set(int option, void* parameter)
 {
 	switch (option) {
-	case UGET_PLUGIN_INIT:
+	case UGET_PLUGIN_GLOBAL_INIT:
 		// do global initialize/uninitialize here
 		if (parameter)
-			return global_init ();
+			return global_init();
 		else
-			global_unref ();
+			global_unref();
 		break;
 
 	default:
@@ -133,15 +133,15 @@ static UgetResult  global_set (int option, void* parameter)
 	return UGET_RESULT_OK;
 }
 
-static UgetResult  global_get (int option, void* parameter)
+static UgetResult  global_get(int option, void* parameter)
 {
 	switch (option) {
-	case UGET_PLUGIN_INIT:
+	case UGET_PLUGIN_GLOBAL_INIT:
 		if (parameter)
 			*(int*)parameter = global.initialized;
 		break;
 
-	case UGET_PLUGIN_ERROR_CODE:
+	case UGET_PLUGIN_GLOBAL_ERROR_CODE:
 		if (parameter)
 			*(int*)parameter = 0;
 		break;
@@ -156,128 +156,133 @@ static UgetResult  global_get (int option, void* parameter)
 // ----------------------------------------------------------------------------
 // control functions
 
-static void plugin_init (UgetPluginEmpty* plugin)
+static void plugin_init(UgetPluginEmpty* plugin)
 {
 	if (global.initialized == FALSE)
-		global_init ();
+		global_init();
 	else
-		global_ref ();
+		global_ref();
 
 	//
 	// your initialized code.
 	//
 }
 
-static void plugin_final (UgetPluginEmpty* plugin)
+static void plugin_final(UgetPluginEmpty* plugin)
 {
 	//
 	// your finalized code.
 	//
 
-	// unassign node
-	if (plugin->node)
-		uget_node_unref (plugin->node);
+	// clear UgData
+	if (plugin->common)
+		ug_group_data_free(plugin->common);
 
-	global_unref ();
+	global_unref();
 }
 
 // ----------------------------------------------------------------------------
 // plugin_ctrl
 
-static int  plugin_ctrl_speed (UgetPluginEmpty* plugin, int* speed);
-static int  plugin_start (UgetPluginEmpty* plugin, UgetNode* node);
-static void plugin_stop (UgetPluginEmpty* plugin);
+static int  plugin_ctrl_speed(UgetPluginEmpty* plugin, int* speed);
+static int  plugin_start(UgetPluginEmpty* plugin);
+static void plugin_stop(UgetPluginEmpty* plugin);
 
-static int  plugin_ctrl (UgetPluginEmpty* plugin, int code, void* data)
+static int  plugin_ctrl(UgetPluginEmpty* plugin, int code, void* data)
 {
 	switch (code) {
 	case UGET_PLUGIN_CTRL_START:
-		if (plugin->node == NULL)
-			return plugin_start (plugin, data);
+		if (plugin->common)
+			return plugin_start(plugin);
 		break;
 
 	case UGET_PLUGIN_CTRL_STOP:
-		plugin_stop (plugin);
+		plugin_stop(plugin);
 		return TRUE;
 
 	case UGET_PLUGIN_CTRL_SPEED:
 		// speed control
-		return plugin_ctrl_speed (plugin, data);
+		return plugin_ctrl_speed(plugin, data);
 
-	// output ---------------
-	case UGET_PLUGIN_CTRL_ACTIVE:
+	// state ----------------
+	case UGET_PLUGIN_GET_STATE:
 		*(int*)data = FALSE;
 		return TRUE;
 
-	// unused ---------------
-	case UGET_PLUGIN_CTRL_NODE_UPDATED:
 	default:
 		break;
 	}
 	return FALSE;
 }
 
-static int  plugin_ctrl_speed (UgetPluginEmpty* plugin, int* speed)
+static int  plugin_ctrl_speed(UgetPluginEmpty* plugin, int* speed)
 {
 	UgetCommon*  common;
 	int          value;
 
-	// Don't do anything if speed limit keep no change.
-	if (plugin->limit[0] == speed[0] && plugin->limit[1] == speed[1])
-		return TRUE;
+	// notify plug-in that speed limit has been changed
+	if (plugin->limit[0] != speed[0] || plugin->limit[1] != speed[1])
+		plugin->limit_changed = TRUE;
 	// decide speed limit by user specified data.
-	if (plugin->node == NULL) {
+	if (plugin->common == NULL) {
 		plugin->limit[0] = speed[0];
 		plugin->limit[1] = speed[1];
 	}
 	else {
-		common = ug_info_realloc (&plugin->node->info, UgetCommonInfo);
+		common = plugin->common;
 		// download
 		value = speed[0];
 		if (common->max_download_speed) {
-			if (value > common->max_download_speed || value == 0)
+			if (value > common->max_download_speed || value == 0) {
 				value = common->max_download_speed;
+				plugin->limit_changed = TRUE;
+			}
 		}
 		plugin->limit[0] = value;
 		// upload
 		value = speed[1];
 		if (common->max_upload_speed) {
-			if (value > common->max_upload_speed || value == 0)
+			if (value > common->max_upload_speed || value == 0) {
 				value = common->max_upload_speed;
+				plugin->limit_changed = TRUE;
+			}
 		}
 		plugin->limit[1] = value;
 	}
-	// notify plug-in that speed limit has been changed
-	plugin->limit_changed = TRUE;
-	return TRUE;
+	return plugin->limit_changed;
 }
 
 // ----------------------------------------------------------------------------
-// plugin_sync
+// plugin_accept/plugin_sync
 
-static int  plugin_sync (UgetPluginEmpty* plugin)
+// return TRUE  if UgData was accepted by plug-in.
+// return FALSE if UgData is lack of necessary data.
+static int  plugin_accept(UgetPluginEmpty* plugin, UgData* data)
+{
+	UgetCommon*  common;
+
+	common = ug_data_get(data, UgetCommonInfo);
+	if (common == NULL || common->uri == NULL)
+		return FALSE;
+
+	plugin->common = ug_group_data_copy(plugin->common);
+	return TRUE;
+}
+
+// return TRUE  if plug-in is running or some data need to sync.
+// return FALSE if plug-in was stopped and no data need to sync.
+static int  plugin_sync(UgetPluginEmpty* plugin, UgData* data)
 {
-	// if plug-in was stopped, return FALSE.
 	return FALSE;
 }
 
 // ----------------------------------------------------------------------------
 
-static int  plugin_start (UgetPluginEmpty* plugin, UgetNode* node)
+static int  plugin_start(UgetPluginEmpty* plugin)
 {
-	UgetCommon*  common;
-
-	common = ug_info_get (&node->info, UgetCommonInfo);
-	if (common == NULL || common->uri == NULL)
-		return FALSE;
-
-	// assign node
-	uget_node_ref (node);
-	plugin->node = node;
-
 	return TRUE;
 }
 
-static void plugin_stop (UgetPluginEmpty* plugin)
+static void plugin_stop(UgetPluginEmpty* plugin)
 {
 }
