@@ -43,27 +43,38 @@
 extern "C" {
 #endif
 
-typedef struct	UgetPluginEmpty          UgetPluginEmpty;
+typedef struct	UgetPluginEmpty   UgetPluginEmpty;
+
+extern const    UgetPluginInfo*   UgetPluginEmptyInfo;
 
 typedef enum {
-	UGET_PLUGIN_EMPTY_BEGIN = UGET_PLUGIN_OPTION_DERIVED,    // begin
+	UGET_PLUGIN_EMPTY_GLOBAL = UGET_PLUGIN_GLOBAL_DERIVED,    // begin
 	// your setting ID...
 } UgetPluginEmptyCode;
 
-extern const UgetPluginInfo*  UgetPluginEmptyInfo;
+/* ----------------------------------------------------------------------------
+   UgetPluginEmpty: an empty plug-in. It derived from UgetPlugin.
 
-// ----------------------------------------------------------------------------
-// UgetPluginEmpty: an empty plug-in. It derived from UgetPlugin.
+   UgType
+   |
+   `--- UgetPlugin
+        |
+        `--- UgetPluginEmpty
+ */
 
 struct UgetPluginEmpty
 {
-	UGET_PLUGIN_MEMBERS;               // It derived from UgetPlugin
-//	const UgetPluginInfo*  info;
-//	UgetEvent*    messages;
-//	UgMutex       mutex;
-//	int           ref_count;
+	UGET_PLUGIN_MEMBERS;
+/*	// ------ UgType members ------
+	const UgetPluginInfo*  info;
 
-	UgetNode*     node;
+	// ------ UgetPlugin members ------
+	UgetEvent*    messages;
+	UgMutex       mutex;
+	int           ref_count;
+ */
+
+	UgetCommon*   common;
 
 	// speed limit control
 	// limit[0] = download speed limit
