@@ -194,6 +194,7 @@ static void uget_node_set_by_dataset (UgetNode* node, UgDataset* dataset)
 
 	union {
 		UgetProgress* progress;
+		UgetRelation* relation;
 		UgetCommon*   common;
 		UgetProxy*    proxy;
 		UgetHttp*     http;
@@ -202,28 +203,29 @@ static void uget_node_set_by_dataset (UgetNode* node, UgDataset* dataset)
 
 	old.relation = ug_dataset_get (dataset, UgRelationInfo, 0);
 	if (old.relation) {
+		new.relation = ug_data_realloc (node->data, UgetRelationInfo);
 		if (old.relation->hints & UG_HINT_PAUSED)
-			node->state |= UGET_STATE_PAUSED;
+			new.relation->group |= UGET_GROUP_PAUSED;
 		if (old.relation->hints & UG_HINT_ERROR)
-			node->state |= UGET_STATE_ERROR;
+			new.relation->group |= UGET_GROUP_ERROR;
 		if (old.relation->hints & UG_HINT_COMPLETED)
-			node->state |= UGET_STATE_COMPLETED;
+			new.relation->group |= UGET_GROUP_COMPLETED;
 
 		if (old.relation->hints & UG_HINT_FINISHED)
-			node->state |= UGET_STATE_FINISHED;
+			new.relation->group |= UGET_GROUP_FINISHED;
 		else if (old.relation->hints & UG_HINT_RECYCLED)
-			node->state |= UGET_STATE_RECYCLED;
+			new.relation->group |= UGET_GROUP_RECYCLED;
 		else
-			node->state |= UGET_STATE_QUEUING;
+			new.relation->group |= UGET_GROUP_QUEUING;
 	}
 
 	old.common = ug_dataset_get (dataset, UgCommonInfo, 0);
 	if (old.common) {
-		node->name = old.common->name;
+		new.common = ug_data_realloc (node->data, UgetCommonInfo);
+		new.common->name = old.common->name;
 		old.common->name = NULL;
-		if (node->name == NULL && old.common->file)
-			node->name = ug_strdup (old.common->file);
-		new.common = ug_info_realloc (&node->info, UgetCommonInfo);
+		if (new.common->name == NULL && old.common->file)
+			new.common->name = ug_strdup (old.common->file);
 		new.common->uri = old.common->url;
 		old.common->url = NULL;
 		new.common->mirrors = old.common->mirrors;
@@ -249,7 +251,7 @@ static void uget_node_set_by_dataset (UgetNode* node, UgDataset* dataset)
 
 	old.proxy = ug_dataset_get (dataset, UgProxyInfo, 0);
 	if (old.proxy) {
-		new.proxy = ug_info_realloc (&node->info, UgetProxyInfo);
+		new.proxy = ug_data_realloc (node->data, UgetProxyInfo);
 		new.proxy->host = old.proxy->host;
 		old.proxy->host = NULL;
 		new.proxy->port = old.proxy->port;
@@ -262,7 +264,7 @@ static void uget_node_set_by_dataset (UgetNode* node, UgDataset* dataset)
 
 	old.http = ug_dataset_get (dataset, UgHttpInfo, 0);
 	if (old.http) {
-		new.http = ug_info_realloc (&node->info, UgetHttpInfo);
+		new.http = ug_data_realloc (node->data, UgetHttpInfo);
 		new.http->user = old.http->user;
 		old.http->user = NULL;
 		new.http->password = old.http->password;
@@ -285,7 +287,7 @@ static void uget_node_set_by_dataset (UgetNode* node, UgDataset* dataset)
 
 	old.ftp = ug_dataset_get (dataset, UgFtpInfo, 0);
 	if (old.ftp) {
-		new.ftp = ug_info_realloc (&node->info, UgetFtpInfo);
+		new.ftp = ug_data_realloc (node->data, UgetFtpInfo);
 		new.ftp->user = old.ftp->user;
 		old.ftp->user = NULL;
 		new.ftp->password = old.ftp->password;
@@ -295,7 +297,7 @@ static void uget_node_set_by_dataset (UgetNode* node, UgDataset* dataset)
 
 	old.progress = ug_dataset_get (dataset, UgProgressInfo, 0);
 	if (old.progress) {
-		new.progress = ug_info_realloc (&node->info, UgetProgressInfo);
+		new.progress = ug_data_realloc (node->data, UgetProgressInfo);
 		new.progress->complete = old.progress->complete;
 		new.progress->total = old.progress->total;
 		new.progress->complete = old.progress->complete;
@@ -312,16 +314,17 @@ static UgetNode* uget_node_from_category (UgCategory* category1)
 	GList*        link;
 	UgetNode*     node;
 	UgetNode*     dnode;
+	UgetCommon*   common;
 	UgetCategory* category;
 
 	node = uget_node_new (NULL);
 	uget_node_set_by_dataset (node, category1->defaults);
-	category = ug_info_realloc (&node->info, UgetCategoryInfo);
+	category = ug_data_realloc (node->data, UgetCategoryInfo);
 	category->active_limit   = category1->active_limit;
 	category->finished_limit = category1->finished_limit;
 	category->recycled_limit = category1->recycled_limit;
-	node->type = UGET_NODE_CATEGORY;
-	node->name = category1->name;
+	common = ug_data_realloc(node->data, UgetCommonInfo);
+	common->name = category1->name;
 	category1->name = NULL;
 	// other
 	*(char**)ug_array_alloc (&category->schemes, 1) = ug_strdup ("http");
@@ -333,7 +336,6 @@ static UgetNode* uget_node_from_category (UgCategory* category1)
 	//
 	for (link = category1->indices;  link;  link = link->next) {
 		dnode = uget_node_new (NULL);
-		dnode->type = UGET_NODE_DOWNLOAD;
 		uget_node_set_by_dataset (dnode, (UgDataset*) link->data);
 		uget_node_append (node, dnode);
 	}
