@@ -99,14 +99,14 @@ static void  on_download_cursor_changed (GtkTreeView* view, UgtkApp* app)
 
 	node = app->traveler.download.cursor.node;
 	if (node)
-		node = node->data;
+		node = node->base;
 	ugtk_summary_show (&app->summary, node);
 
 	// UgtkMenubar.download.priority
 	if (node == NULL)
 		priority = UGET_PRIORITY_NORMAL;
 	else {
-		relation = ug_info_get (&node->info, UgetRelationInfo);
+		relation = ug_data_get (node->data, UgetRelationInfo);
 		if (relation)
 			priority = relation->task.priority;
 		else
@@ -482,7 +482,7 @@ static void node_inserted (UgetNode* node, UgetNode* sibling, UgetNode* child)
 	GtkTreeIter  iter;
 	UgtkApp*     app;
 
-	app = node->notification->data;
+	app = node->control->data;
 	if (node == (UgetNode*) app->traveler.category.model->root) {
 		// category inserted
 		path = gtk_tree_path_new_from_indices (
@@ -513,7 +513,7 @@ static void node_removed (UgetNode* node, UgetNode* sibling, UgetNode* child)
 	UgtkApp*     app;
 	int          pos;
 
-	app = node->notification->data;
+	app = node->control->data;
 	if (node == (UgetNode*) app->traveler.category.model->root) {
 		// category removed
 		if (sibling)
@@ -549,7 +549,7 @@ static void node_updated (UgetNode* child)
 	UgtkApp*     app;
 
 	node = child->parent;
-	app = node->notification->data;
+	app = node->control->data;
 	if (node == (UgetNode*) app->traveler.category.model->root) {
 		// category changed
 		path = gtk_tree_path_new_from_indices (
