@@ -280,13 +280,6 @@ static gboolean  ugtk_app_timeout_queuing (UgtkApp* app)
 				app->task.speed.download, app->task.speed.upload);
 	}
 
-	if (app->n_moved > 0) {
-		// refresh other data & status
-		gtk_widget_queue_draw ((GtkWidget*) app->traveler.download.view);
-		gtk_widget_queue_draw ((GtkWidget*) app->traveler.category.view);
-		gtk_widget_queue_draw ((GtkWidget*) app->traveler.state.view);
-	}
-
 	// if current status is "All" or "Active"
 	if (app->traveler.state.cursor.pos == 0 ||
 	    app->traveler.state.cursor.pos == 1)
@@ -295,6 +288,14 @@ static gboolean  ugtk_app_timeout_queuing (UgtkApp* app)
 			ugtk_traveler_restore_selection (&app->traveler);
 	}
 
+	uget_app_trim((UgetApp*) app);
+	if (app->n_moved > 0) {
+		// refresh other data & status
+		gtk_widget_queue_draw ((GtkWidget*) app->traveler.download.view);
+		gtk_widget_queue_draw ((GtkWidget*) app->traveler.category.view);
+		gtk_widget_queue_draw ((GtkWidget*) app->traveler.state.view);
+	}
+
 	app->user_action = FALSE;
 	app->n_moved = 0;   // reset counter
 	n_active_last = n_active;
@@ -313,7 +314,7 @@ static void  on_keep_above_window_show (GtkWindow *window, gpointer  user_data)
 }
 
 static void  ugtk_app_add_uris_quietly (UgtkApp* app,       GList* list,
-                                        UgetNode* infonode, int  nth_category)
+                                        UgData*  node_data, int  nth_category)
 {
 	GList*      link;
 	UgUri       uuri;
@@ -340,16 +341,16 @@ static void  ugtk_app_add_uris_quietly (UgtkApp* app,       GList* list,
 			ug_uri_init (&uuri, link->data);
 			cnode = uget_app_match_category ((UgetApp*) app, &uuri, NULL);
 		}
-		if (cnode == NULL && infonode) {
+		if (cnode == NULL && node_data) {
 			// match category by filename
-			common = ug_info_realloc (&infonode->info, UgetCommonInfo);
+			common = ug_data_realloc(node_data, UgetCommonInfo);
 			if (common && common->file) {
 				ug_uri_init (&uuri, common->file);
 				cnode = uget_app_match_category ((UgetApp*) app, &uuri, NULL);
 			}
 		}
 		if (cnode == NULL) {
-			if (infonode == NULL) {
+			if (node_data == NULL) {
 				cnode = uget_node_nth_child (&app->real,
 						app->setting.clipboard.nth_category);
 			}
@@ -361,15 +362,15 @@ static void  ugtk_app_add_uris_quietly (UgtkApp* app,       GList* list,
 		if (cnode == NULL)
 			cnode = uget_node_nth_child (&app->real, 0);
 		// add download
-		if (infonode == NULL) {
+		if (node_data == NULL) {
 			// add and free URI
 			uget_app_add_download_uri ((UgetApp*) app, link->data, cnode, TRUE);
 			g_free (link->data);
 		}
 		else {
 			dnode = uget_node_new (NULL);
-			ug_info_assign (&dnode->info, &infonode->info, NULL);
-			common = ug_info_realloc (&dnode->info, UgetCommonInfo);
+			ug_data_assign(dnode->data, node_data, NULL);
+			common = ug_data_realloc (dnode->data, UgetCommonInfo);
 			common->uri = link->data;
 			uget_app_add_download ((UgetApp*) app, dnode, cnode, TRUE);
 		}
@@ -378,7 +379,7 @@ static void  ugtk_app_add_uris_quietly (UgtkApp* app,       GList* list,
 }
 
 static void  ugtk_app_add_uris_selected (UgtkApp* app,       GList* list,
-                                         UgetNode* infonode, int  nth_category)
+                                         UgData*  node_data, int  nth_category)
 {
 	UgtkBatchDialog*    bdialog;
 	UgtkSelectorPage*   page;
@@ -394,7 +395,7 @@ static void  ugtk_app_add_uris_selected (UgtkApp* app,       GList* list,
 	}
 
 	// choose title for clipboard or command-line
-	if (infonode == NULL)
+	if (node_data == NULL)
 		title = g_strconcat (UGTK_APP_NAME " - ", _("New from Clipboard"), NULL);
 	else
 		title = g_strconcat (UGTK_APP_NAME " - ", _("New Download"), NULL);
@@ -405,9 +406,9 @@ static void  ugtk_app_add_uris_selected (UgtkApp* app,       GList* list,
 	if (list->next == NULL)
 		ugtk_batch_dialog_disable_batch (bdialog);
 	// apply other setting
-	if (infonode) {
-		ugtk_download_form_set (&bdialog->download, infonode, FALSE);
-		ugtk_proxy_form_set (&bdialog->proxy, infonode, FALSE);
+	if (node_data) {
+		ugtk_download_form_set(&bdialog->download, node_data, FALSE);
+		ugtk_proxy_form_set(&bdialog->proxy, node_data, FALSE);
 	}
 	// add URIs
 	if (list->next == NULL) {
@@ -420,7 +421,7 @@ static void  ugtk_app_add_uris_selected (UgtkApp* app,       GList* list,
 		// all uris in list will be freed.
 		ugtk_batch_dialog_use_selector (bdialog);
 		ugtk_selector_hide_href (&bdialog->selector);
-		if (infonode == NULL)
+		if (node_data == NULL)
 			page = ugtk_selector_add_page (&bdialog->selector, _("Clipboard"));
 		else
 			page = ugtk_selector_add_page (&bdialog->selector, _("Command line"));
@@ -435,15 +436,15 @@ static void  ugtk_app_add_uris_selected (UgtkApp* app,       GList* list,
 	if (cnode == NULL && list->data) {
 		// match category by URI
 		ug_uri_init (&uuri, list->data);
-		if (infonode == NULL)
+		if (node_data == NULL)
 			cnode = uget_app_match_category ((UgetApp*) app, &uuri, NULL);
 		else {
-			common = ug_info_realloc (&infonode->info, UgetCommonInfo);
+			common = ug_data_realloc(node_data, UgetCommonInfo);
 			cnode = uget_app_match_category ((UgetApp*) app, &uuri, common->file);
 		}
 	}
 	if (cnode == NULL) {
-		if (infonode == NULL) {
+		if (node_data == NULL) {
 			cnode = uget_node_nth_child (&app->real,
 					app->setting.clipboard.nth_category);
 		}
@@ -507,7 +508,7 @@ static gboolean  ugtk_app_timeout_rpc (UgtkApp* app)
 {
 	UgetRpcReq*  req;
 	UgetRpcCmd*  cmd;
-	UgetNode*    infonode;
+	UgData*      node_data;
 
 	for (;;) {
 		if (uget_rpc_has_request(app->rpc) == FALSE)
@@ -557,17 +558,17 @@ static gboolean  ugtk_app_timeout_rpc (UgtkApp* app)
 				break;
 
 			// add downloads
-			infonode = uget_node_new (NULL);
-			uget_option_value_to_info (&cmd->value, &infonode->info);
+			node_data = ug_data_new(8, 0);
+			uget_option_value_to_data(&cmd->value, node_data);
 			if (cmd->value.quiet) {
 				ugtk_app_add_uris_quietly (app, (GList*) cmd->uris.head,
-						infonode, cmd->value.category_index);
+						node_data, cmd->value.category_index);
 			}
 			else {
 				ugtk_app_add_uris_selected (app, (GList*) cmd->uris.head,
-						infonode, cmd->value.category_index);
+						node_data, cmd->value.category_index);
 			}
-			uget_node_unref (infonode);
+			ug_data_unref(node_data);
 			break;
 
 		default:
