@@ -71,10 +71,10 @@ void  ugtk_app_init (UgtkApp* app, UgetRpc* rpc)
 	// clipboard
 	ugtk_clipboard_init (&app->clipboard, app->setting.clipboard.pattern);
 	// plug-in initialize
-	uget_plugin_set (UgetPluginCurlInfo,  UGET_PLUGIN_INIT, (void*) TRUE);
-	uget_plugin_set (UgetPluginAria2Info, UGET_PLUGIN_INIT, (void*) TRUE);
-	uget_plugin_set (UgetPluginMediaInfo, UGET_PLUGIN_INIT, (void*) TRUE);
-	uget_plugin_set (UgetPluginMegaInfo,  UGET_PLUGIN_INIT, (void*) TRUE);
+	uget_plugin_global_set(UgetPluginCurlInfo,  UGET_PLUGIN_GLOBAL_INIT, (void*) TRUE);
+	uget_plugin_global_set(UgetPluginAria2Info, UGET_PLUGIN_GLOBAL_INIT, (void*) TRUE);
+	uget_plugin_global_set(UgetPluginMediaInfo, UGET_PLUGIN_GLOBAL_INIT, (void*) TRUE);
+	uget_plugin_global_set(UgetPluginMegaInfo,  UGET_PLUGIN_GLOBAL_INIT, (void*) TRUE);
 	// apply UgtkSetting
 	ugtk_app_set_plugin_setting (app, &app->setting);
 	ugtk_app_set_window_setting (app, &app->setting);
@@ -87,7 +87,7 @@ void  ugtk_app_init (UgtkApp* app, UgetRpc* rpc)
 	ugtk_menubar_sync_category (&app->menubar, app, TRUE);
 
 	app->recent.category_index = 0;
-	app->recent.infonode = uget_node_new (NULL);
+	app->recent.data = ug_data_new(8, 0);
 	// RSS
 	uget_rss_add_builtin (app->rss_builtin, UGET_RSS_STABLE);
 	uget_rss_add_builtin (app->rss_builtin, UGET_RSS_NEWS);
@@ -117,15 +117,16 @@ void  ugtk_app_final (UgtkApp* app)
 		shutdown_now = app->setting.aria2.shutdown;
 	else
 		shutdown_now = FALSE;
+	ug_data_unref(app->recent.data);
 	uget_rss_unref (app->rss_builtin);
 	uget_app_final ((UgetApp*) app);
 	// plug-in finalize
-	uget_plugin_set (UgetPluginAria2Info, UGET_PLUGIN_ARIA2_SHUTDOWN_NOW,
+	uget_plugin_global_set(UgetPluginAria2Info, UGET_PLUGIN_ARIA2_GLOBAL_SHUTDOWN_NOW,
 			(void*)(intptr_t) shutdown_now);
-	uget_plugin_set (UgetPluginCurlInfo,  UGET_PLUGIN_INIT, (void*) FALSE);
-	uget_plugin_set (UgetPluginAria2Info, UGET_PLUGIN_INIT, (void*) FALSE);
-	uget_plugin_set (UgetPluginMediaInfo, UGET_PLUGIN_INIT, (void*) FALSE);
-	uget_plugin_set (UgetPluginMegaInfo,  UGET_PLUGIN_INIT, (void*) FALSE);
+	uget_plugin_global_set(UgetPluginCurlInfo,  UGET_PLUGIN_GLOBAL_INIT, (void*) FALSE);
+	uget_plugin_global_set(UgetPluginAria2Info, UGET_PLUGIN_GLOBAL_INIT, (void*) FALSE);
+	uget_plugin_global_set(UgetPluginMediaInfo, UGET_PLUGIN_GLOBAL_INIT, (void*) FALSE);
+	uget_plugin_global_set(UgetPluginMegaInfo,  UGET_PLUGIN_GLOBAL_INIT, (void*) FALSE);
 }
 
 void  ugtk_app_save (UgtkApp* app)
@@ -516,37 +517,36 @@ void  ugtk_app_set_plugin_setting (UgtkApp* app, UgtkSetting* setting)
 
 	// set default plug-in
 	uget_app_set_default_plugin ((UgetApp*) app, default_plugin);
+	// set agent plug-in (used by media and MEGA plug-in)
+	uget_plugin_agent_global_set(UGET_PLUGIN_AGENT_GLOBAL_PLUGIN,
+	                 (void*) default_plugin);
 	// set media plug-in
 	uget_app_add_plugin ((UgetApp*) app, UgetPluginMediaInfo);
-	uget_plugin_set (UgetPluginMediaInfo, UGET_PLUGIN_MEDIA_DEFAULT_PLUGIN,
-	                 (void*) default_plugin);
-	uget_plugin_set (UgetPluginMediaInfo, UGET_PLUGIN_MEDIA_MATCH_MODE,
+	uget_plugin_global_set(UgetPluginMediaInfo, UGET_PLUGIN_MEDIA_GLOBAL_MATCH_MODE,
 	                 (void*)(intptr_t) setting->media.match_mode);
-	uget_plugin_set (UgetPluginMediaInfo, UGET_PLUGIN_MEDIA_QUALITY,
+	uget_plugin_global_set(UgetPluginMediaInfo, UGET_PLUGIN_MEDIA_GLOBAL_QUALITY,
 	                 (void*)(intptr_t) setting->media.quality);
-	uget_plugin_set (UgetPluginMediaInfo, UGET_PLUGIN_MEDIA_TYPE,
+	uget_plugin_global_set(UgetPluginMediaInfo, UGET_PLUGIN_MEDIA_GLOBAL_TYPE,
 	                 (void*)(intptr_t) setting->media.type);
-	// set agent plug-in (used by MEGA)
-	uget_plugin_agent_global_set (UGET_PLUGIN_AGENT_DEFAULT_PLUGIN,
-	                 (void*) default_plugin);
+	// set MEGA plug-in
 	uget_app_add_plugin ((UgetApp*) app, UgetPluginMegaInfo);
 	// set aria2 plug-in
 	if (setting->plugin_order >= UGTK_PLUGIN_ORDER_ARIA2) {
-		uget_plugin_set (UgetPluginAria2Info, UGET_PLUGIN_ARIA2_URI,
+		uget_plugin_global_set(UgetPluginAria2Info, UGET_PLUGIN_ARIA2_GLOBAL_URI,
 		                 setting->aria2.uri);
-		uget_plugin_set (UgetPluginAria2Info, UGET_PLUGIN_ARIA2_PATH,
+		uget_plugin_global_set(UgetPluginAria2Info, UGET_PLUGIN_ARIA2_GLOBAL_PATH,
 		                 setting->aria2.path);
-		uget_plugin_set (UgetPluginAria2Info, UGET_PLUGIN_ARIA2_ARGUMENT,
+		uget_plugin_global_set(UgetPluginAria2Info, UGET_PLUGIN_ARIA2_GLOBAL_ARGUMENT,
 		                 setting->aria2.args);
-		uget_plugin_set (UgetPluginAria2Info, UGET_PLUGIN_ARIA2_TOKEN,
+		uget_plugin_global_set(UgetPluginAria2Info, UGET_PLUGIN_ARIA2_GLOBAL_TOKEN,
 		                 setting->aria2.token);
-		uget_plugin_set (UgetPluginAria2Info, UGET_PLUGIN_ARIA2_LAUNCH,
+		uget_plugin_global_set(UgetPluginAria2Info, UGET_PLUGIN_ARIA2_GLOBAL_LAUNCH,
 		                 (void*)(intptr_t) setting->aria2.launch);
-		uget_plugin_set (UgetPluginAria2Info, UGET_PLUGIN_ARIA2_SHUTDOWN,
+		uget_plugin_global_set(UgetPluginAria2Info, UGET_PLUGIN_ARIA2_GLOBAL_SHUTDOWN,
 		                 (void*)(intptr_t) setting->aria2.shutdown);
 		limit[0] = setting->aria2.limit.download * 1024;
 		limit[1] = setting->aria2.limit.upload * 1024;
-		uget_plugin_set (UgetPluginAria2Info, UGET_PLUGIN_SPEED_LIMIT, limit);
+		uget_plugin_global_set(UgetPluginAria2Info, UGET_PLUGIN_GLOBAL_SPEED_LIMIT, limit);
 	}
 
 //	app->aria2.remote_updated = FALSE;
@@ -874,6 +874,8 @@ void  ugtk_app_decide_to_quit (UgtkApp* app)
 void  ugtk_app_create_category (UgtkApp* app)
 {
 	UgtkNodeDialog*  ndialog;
+	UgetCommon* common_src;
+	UgetCommon* common;
 	UgetNode*  cnode_src;
 	UgetNode*  cnode;
 	gchar*     title;
@@ -884,14 +886,18 @@ void  ugtk_app_create_category (UgtkApp* app)
 	ugtk_download_form_set_folders (&ndialog->download, &app->setting);
 
 	// category list
-	cnode_src = app->traveler.category.cursor.node->data;
+	cnode_src = app->traveler.category.cursor.node->base;
 	if (cnode_src->parent != &app->real)
 		cnode_src = app->real.children;
+	common_src = ug_data_get(cnode_src->data, UgetCommonInfo);
 	cnode = uget_node_new (NULL);
-	cnode->name = ug_strdup_printf ("%s%s", _("Copy - "), cnode_src->name);
-	ug_info_assign (&cnode->info, &cnode_src->info, NULL);
+	common = ug_data_realloc(cnode->data, UgetCommonInfo);
+	ug_data_assign (cnode->data, cnode_src->data, NULL);
+	ug_free(common->name);
+	common->name = ug_strdup_printf("%s%s", _("Copy - "),
+						(common_src->name) ? common_src->name : NULL);
 
-	ugtk_node_dialog_set (ndialog, cnode);
+	ugtk_node_dialog_set (ndialog, cnode->data);
 	ugtk_node_dialog_run (ndialog, UGTK_NODE_DIALOG_NEW_CATEGORY, cnode);
 }
 
@@ -915,7 +921,7 @@ void  ugtk_app_create_download (UgtkApp* app, const char* sub_title, const char*
 	ugtk_download_form_set_folders (&ndialog->download, &app->setting);
 
 	// category list
-	cnode = app->traveler.category.cursor.node->data;
+	cnode = app->traveler.category.cursor.node->base;
 	if (cnode->parent != &app->real)
 		cnode = app->real.children;
 
@@ -944,7 +950,7 @@ void  ugtk_app_create_download (UgtkApp* app, const char* sub_title, const char*
 	}
 
 	if (cnode)
-		cnode = cnode->data;
+		cnode = cnode->base;
 	ugtk_node_dialog_set_category (ndialog, cnode);
 	ugtk_node_dialog_run (ndialog, UGTK_NODE_DIALOG_NEW_DOWNLOAD, NULL);
 }
@@ -957,7 +963,7 @@ void  ugtk_app_delete_category (UgtkApp* app)
 	UgetNode*    cnode;
 	int          pos;
 
-	cnode = app->traveler.category.cursor.node->data;
+	cnode = app->traveler.category.cursor.node->base;
 	pos   = app->traveler.category.cursor.pos;
 	// move cursor
 	if (pos <= 0)
@@ -999,11 +1005,11 @@ void  ugtk_app_delete_download (UgtkApp* app, gboolean delete_files)
 
 	cursor = app->traveler.download.cursor.node;
 	if (cursor)
-		cursor = cursor->data;
+		cursor = cursor->base;
 	list = ugtk_traveler_get_selected (&app->traveler);
 	for (link = list;  link;  link = link->next) {
 		node = link->data;
-		node = node->data;
+		node = node->base;
 		link->data = node;
 		if (delete_files || mask & GDK_SHIFT_MASK)
 			uget_app_delete_download ((UgetApp*) app, node, delete_files);
@@ -1048,8 +1054,8 @@ void  ugtk_app_edit_category (UgtkApp* app)
 	ndialog = ugtk_node_dialog_new (title, app, TRUE);
 	g_free (title);
 	ugtk_download_form_set_folders (&ndialog->download, &app->setting);
-	ugtk_node_dialog_set (ndialog, node->data);
-	ugtk_node_dialog_run (ndialog, UGTK_NODE_DIALOG_EDIT_CATEGORY, node->data);
+	ugtk_node_dialog_set (ndialog, node->base->data);
+	ugtk_node_dialog_run (ndialog, UGTK_NODE_DIALOG_EDIT_CATEGORY, node->base);
 }
 
 void  ugtk_app_edit_download (UgtkApp* app)
@@ -1064,8 +1070,8 @@ void  ugtk_app_edit_download (UgtkApp* app)
 	ugtk_download_form_set_folders (&ndialog->download, &app->setting);
 
 	node = app->traveler.download.cursor.node;
-	ugtk_node_dialog_set (ndialog, node->data);
-	ugtk_node_dialog_run (ndialog, UGTK_NODE_DIALOG_EDIT_DOWNLOAD, node->data);
+	ugtk_node_dialog_set (ndialog, node->base->data);
+	ugtk_node_dialog_run (ndialog, UGTK_NODE_DIALOG_EDIT_DOWNLOAD, node->base);
 }
 
 // ------------------------------------
@@ -1073,6 +1079,7 @@ void  ugtk_app_edit_download (UgtkApp* app)
 
 void  ugtk_app_queue_download (UgtkApp* app, gboolean keep_active)
 {
+	UgetRelation* relation;
 	UgetNode*  node;
 	UgetNode*  cursor;
 	GList*     list;
@@ -1080,14 +1087,15 @@ void  ugtk_app_queue_download (UgtkApp* app, gboolean keep_active)
 
 	cursor = app->traveler.download.cursor.node;
 	if (cursor)
-		cursor = cursor->data;
+		cursor = cursor->base;
 
 	list = ugtk_traveler_get_selected (&app->traveler);
 	for (link = list;  link;  link = link->next) {
 		node = link->data;
-		node = node->data;
+		node = node->base;
 		link->data = node;
-		if (keep_active && node->state & UGET_STATE_ACTIVE)
+		relation = ug_data_realloc(node->data, UgetRelationInfo);
+		if (keep_active && relation->group & UGET_GROUP_ACTIVE)
 			continue;
 		uget_app_queue_download ((UgetApp*) app, node);
 	}
@@ -1112,12 +1120,12 @@ void  ugtk_app_pause_download (UgtkApp* app)
 
 	cursor = app->traveler.download.cursor.node;
 	if (cursor)
-		cursor = cursor->data;
+		cursor = cursor->base;
 
 	list = ugtk_traveler_get_selected (&app->traveler);
 	for (link = list;  link;  link = link->next) {
 		node = link->data;
-		node = node->data;
+		node = node->base;
 		link->data = node;
 		uget_app_pause_download ((UgetApp*) app, node);
 	}
@@ -1137,6 +1145,7 @@ void  ugtk_app_pause_download (UgtkApp* app)
 
 void  ugtk_app_switch_download_state (UgtkApp* app)
 {
+	UgetRelation* relation;
 	UgetNode*  node;
 	UgetNode*  cursor;
 	GList*     list;
@@ -1144,16 +1153,17 @@ void  ugtk_app_switch_download_state (UgtkApp* app)
 
 	cursor = app->traveler.download.cursor.node;
 	if (cursor)
-		cursor = cursor->data;
+		cursor = cursor->base;
 
 	list = ugtk_traveler_get_selected (&app->traveler);
 	for (link = list;  link;  link = link->next) {
 		node = link->data;
-		node = node->data;
+		node = node->base;
 		link->data = node;
-		if (node->state & UGET_STATE_PAUSED)
+		relation = ug_data_realloc(node->data, UgetRelationInfo);
+		if (relation->group & UGET_GROUP_PAUSED)
 			uget_app_queue_download ((UgetApp*) app, node);
-		else if (node->state & UGET_STATE_ACTIVE)
+		else if (relation->group & UGET_GROUP_ACTIVE)
 			uget_app_pause_download ((UgetApp*) app, node);
 	}
 	if (app->traveler.state.cursor.pos == 0) {
@@ -1239,17 +1249,17 @@ void  ugtk_app_move_download_to (UgtkApp* app, UgetNode* cnode)
 	GList*    link;
 	GList*    list = NULL;
 
-	if (cnode == app->traveler.category.cursor.node->data)
+	if (cnode == app->traveler.category.cursor.node->base)
 		return;
 
 	list = ugtk_traveler_get_selected (&app->traveler);
 	for (link = list;  link;  link = link->next) {
 		node = link->data;
-		node = node->data;
+		node = node->base;
 		if (node->parent == cnode)
 			continue;
 		uget_node_remove (node->parent, node);
-		uget_node_unref_fake (node);
+		uget_node_clear_fake (node);
 		uget_node_append (cnode, node);
 	}
 	g_list_free (list);
@@ -1379,7 +1389,7 @@ static void  on_save_category_response (GtkWidget* dialog, gint response, UgtkAp
 //	file = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
 	gtk_widget_destroy (dialog);
 	cnode = app->traveler.category.cursor.node;
-	if (uget_app_save_category ((UgetApp*) app, cnode->data, file, NULL) == FALSE)
+	if (uget_app_save_category ((UgetApp*) app, cnode->base, file, NULL) == FALSE)
 		ugtk_app_show_message (app, GTK_MESSAGE_ERROR, _("Failed to save category file."));
 	g_free (file);
 }
@@ -1478,7 +1488,7 @@ static void  on_import_html_file_response (GtkWidget* dialog, gint response, Ugt
 	ugtk_download_form_set_folders (&bdialog->download, &app->setting);
 	ugtk_batch_dialog_use_selector (bdialog);
 	// category
-	cnode = app->traveler.category.cursor.node->data;
+	cnode = app->traveler.category.cursor.node->base;
 	if (cnode->parent != &app->real)
 		cnode = app->real.children;
 	ugtk_batch_dialog_set_category (bdialog, cnode);
@@ -1533,7 +1543,7 @@ static void  on_import_text_file_response (GtkWidget* dialog, gint response, Ugt
 	ugtk_batch_dialog_use_selector (bdialog);
 	ugtk_download_form_set_folders (&bdialog->download, &app->setting);
 	// category
-	cnode = app->traveler.category.cursor.node->data;
+	cnode = app->traveler.category.cursor.node->base;
 	if (cnode->parent != &app->real)
 		cnode = app->real.children;
 	ugtk_batch_dialog_set_category (bdialog, cnode);
@@ -1563,9 +1573,9 @@ static void  on_export_text_file_response (GtkWidget* dialog, gint response, Ugt
 	channel = g_io_channel_new_file (fname, "w", NULL);
 	g_free (fname);
 
-	node = app->traveler.category.cursor.node->data;
+	node = app->traveler.category.cursor.node->base;
 	for (node = node->children;  node;  node = node->next) {
-		common = ug_info_get (&node->data->info, UgetCommonInfo);
+		common = ug_data_get (node->data, UgetCommonInfo);
 		if (common == NULL)
 			continue;
 		if (common->uri) {
@@ -1641,7 +1651,7 @@ void  ugtk_app_sequence_batch (UgtkApp* app)
 	ugtk_download_form_set_folders (&bdialog->download, &app->setting);
 
 	// category list
-	cnode = app->traveler.category.cursor.node->data;
+	cnode = app->traveler.category.cursor.node->base;
 	if (cnode->parent != &app->real)
 		cnode = app->real.children;
 	ugtk_batch_dialog_set_category (bdialog, cnode);
@@ -1686,7 +1696,7 @@ void  ugtk_app_clipboard_batch (UgtkApp* app)
 	g_list_free (list);
 
 	// category list
-	cnode = app->traveler.category.cursor.node->data;
+	cnode = app->traveler.category.cursor.node->base;
 	if (cnode->parent != &app->real)
 		cnode = app->real.children;
 	ugtk_batch_dialog_set_category (bdialog, cnode);
@@ -1763,10 +1773,10 @@ void  ugtk_app_add_default_category (UgtkApp* app)
 	static int    counts = 0;
 
 	cnode = uget_node_new (NULL);
-	cnode->name = ug_strdup_printf ("%s %d", _("New"), counts++);
-	common = ug_info_realloc (&cnode->info, UgetCommonInfo);
+	common = ug_data_realloc (cnode->data, UgetCommonInfo);
+	common->name = ug_strdup_printf ("%s %d", _("New"), counts++);
 	common->folder = ug_strdup (g_get_home_dir ());
-	category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
+	category = ug_data_realloc (cnode->data, UgetCategoryInfo);
 	*(char**)ug_array_alloc (&category->schemes, 1) = ug_strdup ("ftps");
 	*(char**)ug_array_alloc (&category->schemes, 1) = ug_strdup ("magnet");
 	*(char**)ug_array_alloc (&category->hosts, 1) = ug_strdup (".edu");
