@@ -49,26 +49,26 @@
 #if GTK_MAJOR_VERSION >= 3 && GTK_MINOR_VERSION >= 10
 static const UgPair state_icon_pair[] =
 {
-	{(void*)(intptr_t) UGET_STATE_FINISHED,  "go-last"},
-	{(void*)(intptr_t) UGET_STATE_RECYCLED,  "list-remove"},
-	{(void*)(intptr_t) UGET_STATE_PAUSED,    "media-playback-pause"},
-	{(void*)(intptr_t) UGET_STATE_ERROR,     "dialog-error"},
-	{(void*)(intptr_t) UGET_STATE_UPLOADING, "go-up"},
-	{(void*)(intptr_t) UGET_STATE_COMPLETED, "gtk-yes"},
-	{(void*)(intptr_t) UGET_STATE_QUEUING,   "text-x-generic"},
-	{(void*)(intptr_t) UGET_STATE_ACTIVE,    "media-playback-start"},
+	{(void*)(intptr_t) UGET_GROUP_FINISHED,  "go-last"},
+	{(void*)(intptr_t) UGET_GROUP_RECYCLED,  "list-remove"},
+	{(void*)(intptr_t) UGET_GROUP_PAUSED,    "media-playback-pause"},
+	{(void*)(intptr_t) UGET_GROUP_ERROR,     "dialog-error"},
+	{(void*)(intptr_t) UGET_GROUP_UPLOADING, "go-up"},
+	{(void*)(intptr_t) UGET_GROUP_COMPLETED, "gtk-yes"},
+	{(void*)(intptr_t) UGET_GROUP_QUEUING,   "text-x-generic"},
+	{(void*)(intptr_t) UGET_GROUP_ACTIVE,    "media-playback-start"},
 };
 #else
 static const UgPair state_icon_pair[] =
 {
-	{(void*)(intptr_t) UGET_STATE_FINISHED,  GTK_STOCK_GOTO_LAST},
-	{(void*)(intptr_t) UGET_STATE_RECYCLED,  GTK_STOCK_DELETE},
-	{(void*)(intptr_t) UGET_STATE_PAUSED,    GTK_STOCK_MEDIA_PAUSE},
-	{(void*)(intptr_t) UGET_STATE_ERROR,     GTK_STOCK_DIALOG_ERROR},
-	{(void*)(intptr_t) UGET_STATE_UPLOADING, GTK_STOCK_GO_UP},
-	{(void*)(intptr_t) UGET_STATE_COMPLETED, GTK_STOCK_YES},
-	{(void*)(intptr_t) UGET_STATE_QUEUING,   GTK_STOCK_FILE},
-	{(void*)(intptr_t) UGET_STATE_ACTIVE,    GTK_STOCK_MEDIA_PLAY},
+	{(void*)(intptr_t) UGET_GROUP_FINISHED,  GTK_STOCK_GOTO_LAST},
+	{(void*)(intptr_t) UGET_GROUP_RECYCLED,  GTK_STOCK_DELETE},
+	{(void*)(intptr_t) UGET_GROUP_PAUSED,    GTK_STOCK_MEDIA_PAUSE},
+	{(void*)(intptr_t) UGET_GROUP_ERROR,     GTK_STOCK_DIALOG_ERROR},
+	{(void*)(intptr_t) UGET_GROUP_UPLOADING, GTK_STOCK_GO_UP},
+	{(void*)(intptr_t) UGET_GROUP_COMPLETED, GTK_STOCK_YES},
+	{(void*)(intptr_t) UGET_GROUP_QUEUING,   GTK_STOCK_FILE},
+	{(void*)(intptr_t) UGET_GROUP_ACTIVE,    GTK_STOCK_MEDIA_PLAY},
 };
 #endif
 static const int state_icon_pair_len = sizeof (state_icon_pair) / sizeof (UgPair);
@@ -80,6 +80,7 @@ static void col_set_icon (GtkTreeViewColumn *tree_column,
                           gpointer           data)
 {
 	UgetNode*      node;
+	UgetRelation*  relation;
 	const gchar*   icon_name;
 	int            key, index;
 
@@ -89,7 +90,7 @@ static void col_set_icon (GtkTreeViewColumn *tree_column,
 	if (node == NULL)
 		return;
 
-	node = node->data;
+	node = node->base;
 #if GTK_MAJOR_VERSION >= 3 && GTK_MINOR_VERSION >= 10
 	icon_name = "text-x-generic";
 #else
@@ -98,7 +99,8 @@ static void col_set_icon (GtkTreeViewColumn *tree_column,
 	// select icon_name
 	for (index = 0;  index < state_icon_pair_len;  index++) {
 		key = (intptr_t)state_icon_pair[index].key;
-		if ((key & node->state) == key) {
+		relation = ug_data_realloc(node->data, UgetRelationInfo);
+		if ((key & relation->group) == key) {
 			icon_name = state_icon_pair[index].data;
 			break;
 		}
@@ -139,13 +141,14 @@ static void col_set_name (GtkTreeViewColumn *tree_column,
 //		}
 //	}
 
-	node = node->data;
+	node = node->base;
 	name = _("unnamed");
+	common = ug_data_get(node->data, UgetCommonInfo);
 
-	if (node->name)
-		name = node->name;
+	if (common && common->name)
+		name = common->name;
 	else {
-		common = ug_info_get (&node->info, UgetCommonInfo);
+		common = ug_data_get (node->data, UgetCommonInfo);
 		if (common) {
 			if (common->file)
 				name = common->file;
@@ -171,9 +174,9 @@ static void col_set_complete (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	progress = ug_info_get (&node->info, UgetProgressInfo);
+	progress = ug_data_get (node->data, UgetProgressInfo);
 	if (progress && progress->total)
 		string = ug_str_from_int_unit (progress->complete, NULL);
 	else
@@ -198,9 +201,9 @@ static void col_set_total (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	progress = ug_info_get (&node->info, UgetProgressInfo);
+	progress = ug_data_get (node->data, UgetProgressInfo);
 	if (progress && progress->total)
 		string = ug_str_from_int_unit (progress->total, NULL);
 	else
@@ -225,9 +228,9 @@ static void col_set_percent (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	progress = ug_info_get (&node->info, UgetProgressInfo);
+	progress = ug_data_get (node->data, UgetProgressInfo);
 	if (progress && progress->total) {
 		string = ug_strdup_printf ("%d%c", progress->percent, '%');
 		g_object_set (cell, "visible", TRUE, NULL);
@@ -258,9 +261,9 @@ static void col_set_elapsed (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	progress = ug_info_get (&node->info, UgetProgressInfo);
+	progress = ug_data_get (node->data, UgetProgressInfo);
 	if (progress)
 		string = ug_str_from_seconds ((int) progress->elapsed, TRUE);
 	else
@@ -287,10 +290,10 @@ static void col_set_left (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	progress = ug_info_get (&node->info, UgetProgressInfo);
-	relation = ug_info_get (&node->info, UgetRelationInfo);
+	progress = ug_data_get (node->data, UgetProgressInfo);
+	relation = ug_data_get (node->data, UgetRelationInfo);
 
 	if (progress && relation && relation->task.plugin)
 		string = ug_str_from_seconds ((int) progress->left, TRUE);
@@ -317,10 +320,10 @@ static void col_set_speed (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	progress = ug_info_get (&node->info, UgetProgressInfo);
-	relation = ug_info_get (&node->info, UgetRelationInfo);
+	progress = ug_data_get (node->data, UgetProgressInfo);
+	relation = ug_data_get (node->data, UgetRelationInfo);
 
 	if (progress && relation && relation->task.plugin)
 		string = ug_str_from_int_unit (progress->download_speed, "/s");
@@ -347,10 +350,10 @@ static void col_set_upload_speed (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	progress = ug_info_get (&node->info, UgetProgressInfo);
-	relation = ug_info_get (&node->info, UgetRelationInfo);
+	progress = ug_data_get (node->data, UgetProgressInfo);
+	relation = ug_data_get (node->data, UgetRelationInfo);
 
 	if (progress && relation && relation->task.plugin && progress->upload_speed)
 		string = ug_str_from_int_unit (progress->upload_speed, "/s");
@@ -376,9 +379,9 @@ static void col_set_uploaded (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	progress = ug_info_get (&node->info, UgetProgressInfo);
+	progress = ug_data_get (node->data, UgetProgressInfo);
 	if (progress && progress->uploaded)
 		string = ug_str_from_int_unit (progress->uploaded, NULL);
 	else
@@ -403,9 +406,9 @@ static void col_set_ratio (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	progress = ug_info_get (&node->info, UgetProgressInfo);
+	progress = ug_data_get (node->data, UgetProgressInfo);
 	if (progress && progress->ratio)
 		string = ug_strdup_printf ("%.2f", progress->ratio);
 	else
@@ -430,9 +433,9 @@ static void col_set_retry (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	common = ug_info_get (&node->info, UgetCommonInfo);
+	common = ug_data_get (node->data, UgetCommonInfo);
 	if (common == NULL || common->retry_count == 0)
 		string = NULL;
 	else if (common->retry_count < 100)
@@ -452,6 +455,7 @@ static void col_set_category (GtkTreeViewColumn *tree_column,
                               GtkTreeIter       *iter,
                               gpointer           data)
 {
+	UgetCommon*   common;
 	UgetNode*     node;
 	char*         string;
 
@@ -460,10 +464,15 @@ static void col_set_category (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	if (node->parent)
-		string = node->parent->name;
+	if (node->parent) {
+		common = ug_data_get(node->parent->data, UgetCommonInfo);
+		if (common)
+			string = common->name;
+		else
+			string = NULL;
+	}
 	else
 		string = NULL;
 
@@ -485,9 +494,9 @@ static void col_set_uri (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	common = ug_info_get (&node->info, UgetCommonInfo);
+	common = ug_data_get (node->data, UgetCommonInfo);
 	if (common)
 		string = common->uri;
 	else
@@ -511,9 +520,9 @@ static void col_set_added_on (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	ulog = ug_info_get (&node->info, UgetLogInfo);
+	ulog = ug_data_get (node->data, UgetLogInfo);
 	if (ulog && ulog->added_time)
 		string = ug_str_from_time (ulog->added_time, FALSE);
 	else
@@ -538,9 +547,9 @@ static void col_set_completed_on (GtkTreeViewColumn *tree_column,
 	// avoid crash in GTK3
 	if (node == NULL)
 		return;
-	node = node->data;
+	node = node->base;
 
-	ulog = ug_info_get (&node->info, UgetLogInfo);
+	ulog = ug_data_get (node->data, UgetLogInfo);
 	if (ulog && ulog->completed_time)
 		string = ug_str_from_time (ulog->completed_time, FALSE);
 	else
@@ -582,6 +591,7 @@ static void col_set_name_c (GtkTreeViewColumn *tree_column,
                             GtkTreeIter       *iter,
                             gpointer           data)
 {
+	UgetCommon*   common;
 	UgetNode*     node;
 	char*         name;
 
@@ -601,9 +611,10 @@ static void col_set_name_c (GtkTreeViewColumn *tree_column,
 //		}
 //	}
 
-	node = node->data;
-	if (node->name)
-		name = node->name;
+	node = node->base;
+	common = ug_data_get(node->data, UgetCommonInfo);
+	if (common && common->name)
+		name = common->name;
 	else
 		name = _("unnamed");
 
@@ -620,12 +631,12 @@ static void col_set_icon_c (GtkTreeViewColumn *tree_column,
 
 	node = iter->user_data;
 #if GTK_MAJOR_VERSION >= 3 && GTK_MINOR_VERSION >= 10
-	if (node->state & UGET_STATE_PAUSED)
+	if (uget_node_get_group(node) & UGET_GROUP_PAUSED)
 		g_object_set (cell, "icon-name", "media-playback-pause", NULL);
 	else
 		g_object_set (cell, "icon-name", "gtk-dnd-multiple", NULL);
 #else
-	if (node->state & UGET_STATE_PAUSED)
+	if (uget_node_get_group(node) & UGET_GROUP_PAUSED)
 		g_object_set (cell, "stock-id", GTK_STOCK_MEDIA_PAUSE, NULL);
 	else
 		g_object_set (cell, "stock-id", GTK_STOCK_DND_MULTIPLE, NULL);
@@ -637,14 +648,14 @@ static void col_set_icon_c (GtkTreeViewColumn *tree_column,
 
 static const UgPair state_name_pair[] =
 {
-	{(void*)(intptr_t) UGET_STATE_ERROR,     N_("Error")},
-	{(void*)(intptr_t) UGET_STATE_PAUSED,    N_("Paused")},
-	{(void*)(intptr_t) UGET_STATE_UPLOADING, N_("Uploading")},
-	{(void*)(intptr_t) UGET_STATE_COMPLETED, N_("Completed")},
-	{(void*)(intptr_t) UGET_STATE_FINISHED,  N_("Finished")},
-	{(void*)(intptr_t) UGET_STATE_RECYCLED,  N_("Recycled")},
-	{(void*)(intptr_t) UGET_STATE_QUEUING,   N_("Queuing")},
-	{(void*)(intptr_t) UGET_STATE_ACTIVE,    N_("Active")},
+	{(void*)(intptr_t) UGET_GROUP_ERROR,     N_("Error")},
+	{(void*)(intptr_t) UGET_GROUP_PAUSED,    N_("Paused")},
+	{(void*)(intptr_t) UGET_GROUP_UPLOADING, N_("Uploading")},
+	{(void*)(intptr_t) UGET_GROUP_COMPLETED, N_("Completed")},
+	{(void*)(intptr_t) UGET_GROUP_FINISHED,  N_("Finished")},
+	{(void*)(intptr_t) UGET_GROUP_RECYCLED,  N_("Recycled")},
+	{(void*)(intptr_t) UGET_GROUP_QUEUING,   N_("Queuing")},
+	{(void*)(intptr_t) UGET_GROUP_ACTIVE,    N_("Active")},
 };
 static const int state_name_pair_len = sizeof (state_name_pair) / sizeof (UgPair);
 
@@ -658,6 +669,7 @@ static void col_set_name_s (GtkTreeViewColumn *tree_column,
 	char*         name;
 	int           key;
 	int           index;
+	int           group;
 
 //	gtk_tree_model_get (model, iter, 0, &node, -1);
 	node = iter->user_data;
@@ -667,9 +679,10 @@ static void col_set_name_s (GtkTreeViewColumn *tree_column,
 
 	name = _("All Status");
 	if (node->real) {
+		group = uget_node_get_group(node);
 		for (index = 0;  index < state_name_pair_len;  index++) {
 			key = (intptr_t)state_name_pair[index].key;
-			if ((key & node->state) == key) {
+			if ((key & group) == key) {
 				name = gettext (state_name_pair[index].data);
 				break;
 			}
@@ -687,6 +700,7 @@ static void col_set_icon_s (GtkTreeViewColumn *tree_column,
 	UgetNode*      node;
 	const gchar*   icon_name;
 	int            key, index;
+	int            group;
 
 //	gtk_tree_model_get (model, iter, 0, &node, -1);
 	node = iter->user_data;
@@ -701,9 +715,10 @@ static void col_set_icon_s (GtkTreeViewColumn *tree_column,
 	icon_name = GTK_STOCK_DND_MULTIPLE;
 #endif
 	if (node->real) {
+		group = uget_node_get_group(node);
 		for (index = 0;  index < state_icon_pair_len;  index++) {
 			key = (intptr_t)state_icon_pair[index].key;
-			if ((key & node->state) == key) {
+			if ((key & group) == key) {
 				icon_name = state_icon_pair[index].data;
 				break;
 			}
