@@ -91,14 +91,13 @@ void  ugtk_summary_show (UgtkSummary* summary, UgetNode* node)
 	}
 
 	iter.stamp = 0;   // used by ugtk_summary_store_realloc_next()
-	node = node->data;
-	temp.common = ug_info_get (&node->info, UgetCommonInfo);
+	temp.common = ug_data_get (node->data, UgetCommonInfo);
 
 	// Summary Name
 	if (summary->visible.name) {
-		if (node->name) {
+		if (temp.common && temp.common->name) {
 			name = g_strconcat (_("Name"), ":", NULL);
-			value = node->name;
+			value = temp.common->name;
 		}
 		else {
 			name = g_strconcat (_("File"), ":", NULL);
@@ -129,7 +128,13 @@ void  ugtk_summary_show (UgtkSummary* summary, UgetNode* node)
 	// Summary Category
 	if (summary->visible.category) {
 		name = g_strconcat (_("Category"), ":", NULL);
-		value = (node->parent) ? node->parent->name : NULL;
+		if (node->parent) {
+			temp.common = ug_data_get (node->parent->data, UgetCommonInfo);
+			value = (temp.common) ? temp.common->name : NULL;
+			temp.common = ug_data_get (node->data, UgetCommonInfo);
+		}
+		else
+			value = NULL;
 		ugtk_summary_store_realloc_next (summary->store, &iter);
 		gtk_list_store_set (summary->store, &iter,
 				UGTK_SUMMARY_COLUMN_ICON , GTK_STOCK_DND_MULTIPLE,
@@ -151,7 +156,7 @@ void  ugtk_summary_show (UgtkSummary* summary, UgetNode* node)
 		g_free (name);
 	}
 	// Summary Message
-	temp.log = ug_info_get (&node->info, UgetLogInfo);
+	temp.log = ug_data_get (node->data, UgetLogInfo);
 	if (temp.log)
 		temp.event = (UgetEvent*) temp.log->messages.head;
 	if (summary->visible.message) {
